'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var popmotion = require('popmotion');
var easingLookup = require('@popmotion/easing');
var isPropValid = _interopDefault(require('@emotion/is-prop-valid'));
var sync = require('framesync');
var sync__default = _interopDefault(sync);
var styler = require('stylefire');
var styler__default = _interopDefault(styler);
var styleValueTypes = require('style-value-types');
var React = require('react');
var popcorn = require('@popmotion/popcorn');
var heyListen = require('hey-listen');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
}

/**
 * Uses the ref that is passed in, or creates a new one
 * @param external - External ref
 * @internal
 */
function useExternalRef(external) {
    // We're conditionally calling `useRef` here which is sort of naughty as hooks
    // shouldn't be called conditionally. However, Framer Motion will break if this
    // condition changes anyway. It might be possible to use an invariant here to
    // make it explicit, but I expect changing `ref` is not normal behaviour.
    var ref = !external || typeof external === "function" ? React.useRef(null) : external;
    React.useEffect(function () {
        if (external && typeof external === "function") {
            external(ref.current);
            return function () { return external(null); };
        }
    }, []);
    return ref;
}

var isFloat = function (value) {
    return !isNaN(parseFloat(value));
};
/**
 * `MotionValue` is used to track the state and velocity of motion values.
 *
 * @public
 */
var MotionValue = /** @class */ (function () {
    /**
     * @param init - The initiating value
     * @param config - Optional configuration options
     *
     * -  `transformer`: A function to transform incoming values with.
     *
     * @internal
     */
    function MotionValue(init, _a) {
        var _b = _a === void 0 ? {} : _a, transformer = _b.transformer, parent = _b.parent;
        var _this = this;
        /**
         * Duration, in milliseconds, since last updating frame.
         *
         * @internal
         */
        this.timeDelta = 0;
        /**
         * Timestamp of the last time this `MotionValue` was updated.
         *
         * @internal
         */
        this.lastUpdated = 0;
        /**
         * Tracks whether this value can output a velocity. Currently this is only true
         * if the value is numerical, but we might be able to widen the scope here and support
         * other value types.
         *
         * @internal
         */
        this.canTrackVelocity = false;
        /**
         * Notify a subscriber with the latest value.
         *
         * This is an instanced and bound function to prevent generating a new
         * function once per frame.
         *
         * @param subscriber - The subscriber to notify.
         *
         * @internal
         */
        this.notifySubscriber = function (subscriber) {
            subscriber(_this.current);
        };
        /**
         * Schedule a velocity check for the next frame.
         *
         * This is an instanced and bound function to prevent generating a new
         * function once per frame.
         *
         * @internal
         */
        this.scheduleVelocityCheck = function () { return sync__default.postRender(_this.velocityCheck); };
        /**
         * Updates `prev` with `current` if the value hasn't been updated this frame.
         * This ensures velocity calculations return `0`.
         *
         * This is an instanced and bound function to prevent generating a new
         * function once per frame.
         *
         * @internal
         */
        this.velocityCheck = function (_a) {
            var timestamp = _a.timestamp;
            if (timestamp !== _this.lastUpdated) {
                _this.prev = _this.current;
            }
        };
        /**
         * Updates child `MotionValue`.
         *
         * @param child - Child `MotionValue`.
         *
         * @internal
         */
        this.setChild = function (child) { return child.set(_this.current); };
        this.parent = parent;
        this.transformer = transformer;
        this.set(init, false);
        this.canTrackVelocity = isFloat(this.current);
    }
    /**
     * Creates a new `MotionValue` that's subscribed to the output of this one.
     *
     * @param config - Optional configuration options
     *
     * -  `transformer`: A function to transform incoming values with.
     *
     * @internal
     */
    MotionValue.prototype.addChild = function (config) {
        var child = new MotionValue(this.current, __assign({ parent: this }, config));
        if (!this.children)
            this.children = new Set();
        this.children.add(child);
        return child;
    };
    /**
     * Stops a `MotionValue` from being subscribed to this one.
     *
     * @param child - The subscribed `MotionValue`
     *
     * @internal
     */
    MotionValue.prototype.removeChild = function (child) {
        if (!this.children) {
            return;
        }
        this.children.delete(child);
    };
    /**
     * Subscribes a subscriber function to a subscription list.
     *
     * @param subscriptions - A `Set` of subscribers.
     * @param subscription - A subscriber function.
     */
    MotionValue.prototype.subscribeTo = function (subscriptions, subscription) {
        var _this = this;
        var updateSubscriber = function () { return subscription(_this.current); };
        subscriptions.add(updateSubscriber);
        return function () { return subscriptions.delete(updateSubscriber); };
    };
    /**
     * Adds a function that will be notified when the `MotionValue` is updated.
     *
     * It returns a function that, when called, will cancel the subscription.
     *
     * When calling `onChange` inside a React component, it should be wrapped with the
     * `useEffect` hook. As it returns an unsubscribe function, this should be returned
     * from the `useEffect` function to ensure only one active subscription persists
     * for the lifecycle of the component.
     *
     * ```jsx
     * const MyComponent = () => {
     *   const x = useMotionValue(0)
     *
     *   useEffect(() => {
     *     const unsubscribe = x.onChange((latestX) => {
     *       // Do stuff with latest x value
     *     })
     *
     *     return unsubscribe
     *   })
     *
     *   return <motion.div style={{ x }} />
     * }
     * ```
     *
     * @internalremarks
     *
     * We could look into a `useOnChange` hook if the above lifecycle management proves confusing.
     *
     * ```jsx
     * useOnChange(x, () => {})
     * ```
     *
     * @param subscriber - A function that receives the latest value.
     * @returns A function that, when called, will cancel this subscription.
     *
     * @public
     */
    MotionValue.prototype.onChange = function (subscription) {
        if (!this.updateSubscribers)
            this.updateSubscribers = new Set();
        return this.subscribeTo(this.updateSubscribers, subscription);
    };
    /**
     * Adds a function that will be notified when the `MotionValue` requests a render.
     *
     * @param subscriber - A function that's provided the latest value.
     * @returns A function that, when called, will cancel this subscription.
     *
     * @internal
     */
    MotionValue.prototype.onRenderRequest = function (subscription) {
        if (!this.renderSubscribers)
            this.renderSubscribers = new Set();
        // Render immediately
        this.notifySubscriber(subscription);
        return this.subscribeTo(this.renderSubscribers, subscription);
    };
    /**
     * Sets the state of the `MotionValue`.
     *
     * @remarks
     *
     * ```jsx
     * const x = useMotionValue(0)
     * x.set(10)
     * ```
     *
     * @param latest - Latest value to set.
     * @param render - Whether to notify render subscribers. Defaults to `true`
     *
     * @public
     */
    MotionValue.prototype.set = function (v, render) {
        if (render === void 0) { render = true; }
        this.prev = this.current;
        this.current = this.transformer ? this.transformer(v) : v;
        if (this.updateSubscribers && this.prev !== this.current) {
            this.updateSubscribers.forEach(this.notifySubscriber);
        }
        if (this.children) {
            this.children.forEach(this.setChild);
        }
        if (render && this.renderSubscribers) {
            this.renderSubscribers.forEach(this.notifySubscriber);
        }
        // Update timestamp
        var _a = sync.getFrameData(), delta = _a.delta, timestamp = _a.timestamp;
        if (this.lastUpdated !== timestamp) {
            this.timeDelta = delta;
            this.lastUpdated = timestamp;
            sync__default.postRender(this.scheduleVelocityCheck);
        }
    };
    /**
     * Returns the latest state of `MotionValue`
     *
     * @returns - The latest state of `MotionValue`
     *
     * @public
     */
    MotionValue.prototype.get = function () {
        return this.current;
    };
    /**
     * Returns the latest velocity of `MotionValue`
     *
     * @returns - The latest velocity of `MotionValue`. Returns `0` if the state is non-numerical.
     *
     * @public
     */
    MotionValue.prototype.getVelocity = function () {
        // This could be isFloat(this.prev) && isFloat(this.current), but that would be wasteful
        return this.canTrackVelocity
            ? // These casts could be avoided if parseFloat would be typed better
                popcorn.velocityPerSecond(parseFloat(this.current) -
                    parseFloat(this.prev), this.timeDelta)
            : 0;
    };
    /**
     * Registers a new animation to control this `MotionValue`. Only one
     * animation can drive a `MotionValue` at one time.
     *
     * ```jsx
     * value.start()
     * ```
     *
     * @param animation - A function that starts the provided animation
     *
     * @internal
     */
    MotionValue.prototype.start = function (animation) {
        var _this = this;
        this.stop();
        return new Promise(function (resolve) {
            _this.stopAnimation = animation(resolve);
        }).then(function () { return _this.clearAnimation(); });
    };
    /**
     * Stop the currently controlling animation.
     *
     * @public
     */
    MotionValue.prototype.stop = function () {
        if (this.stopAnimation)
            this.stopAnimation();
        this.clearAnimation();
    };
    /**
     * Returns `true` if this value is currently animating.
     *
     * @public
     */
    MotionValue.prototype.isAnimating = function () {
        return !!this.stopAnimation;
    };
    MotionValue.prototype.clearAnimation = function () {
        this.stopAnimation = null;
    };
    /**
     * Destroy and clean up this `MotionValue`.
     *
     * @public
     */
    MotionValue.prototype.destroy = function () {
        this.updateSubscribers && this.updateSubscribers.clear();
        this.renderSubscribers && this.renderSubscribers.clear();
        this.parent && this.parent.removeChild(this);
        this.stop();
    };
    return MotionValue;
}());
/**
 * @internal
 */
function motionValue(init, opts) {
    return new MotionValue(init, opts);
}

/**
 * Creates a constant value over the lifecycle of a component.
 *
 * Even if `useMemo` is provided an empty array as its final argument, it doesn't offer
 * a guarantee that it won't re-run for performance reasons later on. By using `useConstant`
 * you can ensure that initialisers don't execute twice or more.
 */
function useConstant(init) {
    var ref = React.useRef(null);
    if (ref.current === null) {
        ref.current = init();
    }
    return ref.current;
}

// Creating a styler factory for the `onUpdate` prop allows all values
// to fire and the `onUpdate` prop will only fire once per frame
var updateStyler = styler.createStylerFactory({
    onRead: function () { return null; },
    onRender: function (state, _a) {
        var onUpdate = _a.onUpdate;
        return onUpdate(state);
    },
});
var MotionValuesMap = /** @class */ (function () {
    function MotionValuesMap() {
        this.hasMounted = false;
        this.values = new Map();
        this.unsubscribers = new Map();
    }
    MotionValuesMap.prototype.has = function (key) {
        return this.values.has(key);
    };
    MotionValuesMap.prototype.set = function (key, value) {
        this.values.set(key, value);
        if (this.hasMounted) {
            this.bindValueToOutput(key, value);
        }
    };
    MotionValuesMap.prototype.get = function (key, defaultValue) {
        var value = this.values.get(key);
        if (value === undefined && defaultValue !== undefined) {
            value = new MotionValue(defaultValue);
            this.set(key, value);
        }
        return value;
    };
    MotionValuesMap.prototype.forEach = function (callback) {
        return this.values.forEach(callback);
    };
    MotionValuesMap.prototype.bindValueToOutput = function (key, value) {
        var _this = this;
        var onRender = function (v) { return _this.output && _this.output(key, v); };
        var unsubscribeOnRender = value.onRenderRequest(onRender);
        var onChange = function (v) {
            _this.onUpdate && _this.onUpdate.set(key, v);
        };
        var unsubscribeOnChange = value.onChange(onChange);
        this.unsubscribers.set(key, function () {
            unsubscribeOnRender();
            unsubscribeOnChange();
        });
    };
    MotionValuesMap.prototype.setOnUpdate = function (onUpdate) {
        this.onUpdate = undefined;
        if (onUpdate) {
            this.onUpdate = updateStyler({ onUpdate: onUpdate });
        }
    };
    MotionValuesMap.prototype.setTransformTemplate = function (transformTemplate) {
        if (this.transformTemplate !== transformTemplate) {
            this.transformTemplate = transformTemplate;
            this.updateTransformTemplate();
        }
    };
    MotionValuesMap.prototype.getTransformTemplate = function () {
        return this.transformTemplate;
    };
    MotionValuesMap.prototype.updateTransformTemplate = function () {
        if (this.output) {
            this.output("transform", this.transformTemplate);
        }
    };
    MotionValuesMap.prototype.mount = function (output) {
        var _this = this;
        this.hasMounted = true;
        if (output)
            this.output = output;
        this.values.forEach(function (value, key) { return _this.bindValueToOutput(key, value); });
        this.updateTransformTemplate();
    };
    MotionValuesMap.prototype.unmount = function () {
        var _this = this;
        this.values.forEach(function (_value, key) {
            var unsubscribe = _this.unsubscribers.get(key);
            unsubscribe && unsubscribe();
        });
    };
    return MotionValuesMap;
}());
var useMotionValues = function (_a) {
    var onUpdate = _a.onUpdate, transformTemplate = _a.transformTemplate;
    var motionValues = useConstant(function () { return new MotionValuesMap(); });
    motionValues.setOnUpdate(onUpdate);
    motionValues.setTransformTemplate(transformTemplate);
    return motionValues;
};
/**
 * `useEffect` gets resolved bottom-up. We defer some optional functionality to child
 * components, so to ensure everything runs correctly we export the ref-binding logic
 * to a new component rather than in `useMotionValues`.
 */
var MountMotionValuesComponent = function (_a, ref) {
    var values = _a.values, isStatic = _a.isStatic;
    React.useEffect(function () {
        heyListen.invariant(ref.current instanceof Element, "No `ref` found. Ensure components created with `motion.custom` forward refs using `React.forwardRef`");
        var domStyler = styler__default(ref.current, {
            preparseOutput: false,
            enableHardwareAcceleration: !isStatic,
        });
        values.mount(function (key, value) { return domStyler.set(key, value); });
        return function () { return values.unmount(); };
    }, []);
    return null;
};
var MountMotionValues = React.memo(React.forwardRef(MountMotionValuesComponent));

var createValueResolver = function (resolver) { return function (values) {
    var resolvedValues = {};
    values.forEach(function (value, key) { return (resolvedValues[key] = resolver(value)); });
    return resolvedValues;
}; };
var resolveCurrent = createValueResolver(function (value) { return value.get(); });

var transformOriginProps = new Set(["originX", "originY"]);
var isTransformOriginProp = function (key) { return transformOriginProps.has(key); };
var isMotionValue = function (value) {
    return value instanceof MotionValue;
};
var buildStyleAttr = function (values, styleProp, isStatic) {
    var motionValueStyles = resolveCurrent(values);
    var transformTemplate = values.getTransformTemplate();
    if (transformTemplate) {
        // If `transform` has been manually set as a string, pass that through the template
        // otherwise pass it forward to Stylefire's style property builder
        motionValueStyles.transform = styleProp.transform
            ? transformTemplate({}, styleProp.transform)
            : transformTemplate;
    }
    return __assign({}, styleProp, styler.buildStyleProperty(motionValueStyles, !isStatic));
};
var useMotionStyles = function (values, styleProp, transformValues) {
    if (styleProp === void 0) { styleProp = {}; }
    var style = React.useRef({}).current;
    var prevMotionStyles = React.useRef({}).current;
    var currentStyleKeys = new Set(Object.keys(style));
    for (var key in styleProp) {
        currentStyleKeys.delete(key);
        var thisStyle = styleProp[key];
        if (isMotionValue(thisStyle)) {
            // If this is a motion value, add it to our MotionValuesMap
            values.set(key, thisStyle);
        }
        else if (styler.isTransformProp(key) || isTransformOriginProp(key)) {
            // Or if it's a transform prop, create a motion value (or update an existing one)
            // to ensure Stylefire can reconcile all the transform values together.
            if (!values.has(key)) {
                // If it doesn't exist as a motion value, create it
                values.set(key, motionValue(thisStyle));
            }
            else {
                // Otherwise only update it if it's changed from a previous render
                if (thisStyle !== prevMotionStyles[key]) {
                    var value = values.get(key);
                    value.set(thisStyle);
                }
            }
            prevMotionStyles[key] = thisStyle;
        }
        else {
            style[key] = thisStyle;
        }
    }
    currentStyleKeys.forEach(function (key) { return delete style[key]; });
    return transformValues ? transformValues(style) : style;
};

var isKeyframesTarget = function (v) {
    return Array.isArray(v);
};

var isCustomValue = function (v) {
    return Boolean(v && typeof v === "object" && v.mix && v.toValue);
};
var resolveFinalValueInKeyframes = function (v) {
    // TODO maybe throw if v.length - 1 is placeholder token?
    return isKeyframesTarget(v) ? v[v.length - 1] || 0 : v;
};

var auto = {
    test: function (v) { return v === "auto"; },
    parse: function (v) { return v; },
};
var dimensionTypes = [styleValueTypes.number, styleValueTypes.px, styleValueTypes.percent, styleValueTypes.degrees, styleValueTypes.vw, styleValueTypes.vh, auto];
var valueTypes = dimensionTypes.concat([styleValueTypes.color, styleValueTypes.complex]);
var testValueType = function (v) { return function (type) { return type.test(v); }; };
var getDimensionValueType = function (v) {
    return dimensionTypes.find(testValueType(v));
};
var getValueType = function (v) { return valueTypes.find(testValueType(v)); };

var underDampedSpring = function () { return ({
    type: "spring",
    stiffness: 500,
    damping: 25,
    restDelta: 0.5,
    restSpeed: 10,
}); };
var overDampedSpring = function (to) { return ({
    type: "spring",
    stiffness: 700,
    damping: to === 0 ? 100 : 35,
}); };
var linearTween = function () { return ({
    ease: "linear",
    duration: 0.3,
}); };
var keyframes = function (values) { return ({
    type: "keyframes",
    duration: 0.8,
    values: values,
}); };
var defaultTransitions = {
    x: underDampedSpring,
    y: underDampedSpring,
    z: underDampedSpring,
    rotate: underDampedSpring,
    rotateX: underDampedSpring,
    rotateY: underDampedSpring,
    rotateZ: underDampedSpring,
    scaleX: overDampedSpring,
    scaleY: overDampedSpring,
    scale: overDampedSpring,
    opacity: linearTween,
    backgroundColor: linearTween,
    color: linearTween,
    default: overDampedSpring,
};
var getDefaultTransition = function (valueKey, to) {
    var transitionFactory;
    if (isKeyframesTarget(to)) {
        transitionFactory = keyframes;
    }
    else {
        transitionFactory =
            defaultTransitions[valueKey] || defaultTransitions.default;
    }
    return __assign({ to: to }, transitionFactory(to));
};

/**
 * A Popmotion action that accepts a single `to` prop. When it starts, it immediately
 * updates with `to` and then completes. By using this we can compose instant transitions
 * in with the same logic that applies `delay` or returns a `Promise` etc.
 */
var just = function (_a) {
    var to = _a.to;
    return popmotion.action(function (_a) {
        var update = _a.update, complete = _a.complete;
        update(to);
        complete();
    });
};

var easingDefinitionToFunction = function (definition) {
    if (Array.isArray(definition)) {
        // If cubic bezier definition, create bezier curve
        heyListen.invariant(definition.length === 4, "Cubic bezier arrays must contain four numerical values.");
        var x1 = definition[0], y1 = definition[1], x2 = definition[2], y2 = definition[3];
        return easingLookup.cubicBezier(x1, y1, x2, y2);
    }
    else if (typeof definition === "string") {
        // Else lookup from table
        heyListen.invariant(easingLookup[definition] !== undefined, "Invalid easing type '" + definition + "'");
        return easingLookup[definition];
    }
    return definition;
};
var isEasingArray = function (ease) {
    return Array.isArray(ease) && typeof ease[0] !== "number";
};

var isDurationAnimation = function (v) { return v.hasOwnProperty("duration"); };

/**
 * Check if a value is animatable. Examples:
 *
 * ✅: 100, "100px", "#fff"
 * ❌: "block", "url(2.jpg)"
 * @param value
 *
 * @internal
 */
var isAnimatable = function (key, value) {
    // If the list of keys tat might be non-animatable grows, replace with Set
    if (key === "zIndex")
        return false;
    // If it's a number or a keyframes array, we can animate it. We might at some point
    // need to do a deep isAnimatable check of keyframes, or let Popmotion handle this,
    // but for now lets leave it like this for performance reasons
    if (typeof value === "number" || Array.isArray(value))
        return true;
    if (typeof value === "string" && // It's animatable if we have a string
        styleValueTypes.complex.test(value) && // And it contains numbers and/or colors
        !value.startsWith("url(") // Unless it starts with "url("
    ) {
        return true;
    }
    return false;
};

var transitions = { tween: popmotion.tween, spring: popmotion.spring, keyframes: popmotion.keyframes, inertia: popmotion.inertia, just: just };
var transitionOptionParser = {
    tween: function (opts) {
        if (opts.ease) {
            var ease = isEasingArray(opts.ease) ? opts.ease[0] : opts.ease;
            opts.ease = easingDefinitionToFunction(ease);
        }
        return opts;
    },
    keyframes: function (_a) {
        var from = _a.from, to = _a.to, velocity = _a.velocity, opts = __rest(_a, ["from", "to", "velocity"]);
        if (opts.values && opts.values[0] === null) {
            var values = opts.values.slice();
            values[0] = from;
            opts.values = values;
        }
        if (opts.ease) {
            opts.easings = isEasingArray(opts.ease)
                ? opts.ease.map(easingDefinitionToFunction)
                : easingDefinitionToFunction(opts.ease);
        }
        opts.ease = easingLookup.linear;
        return opts;
    },
};
var isTransitionDefined = function (_a) {
    var when = _a.when, delay = _a.delay, delayChildren = _a.delayChildren, staggerChildren = _a.staggerChildren, staggerDirection = _a.staggerDirection, transition = __rest(_a, ["when", "delay", "delayChildren", "staggerChildren", "staggerDirection"]);
    return Object.keys(transition).length;
};
var getTransitionDefinition = function (key, to, transitionDefinition) {
    var delay = transitionDefinition ? transitionDefinition.delay : 0;
    // If no object, return default transition
    // A better way to handle this would be to deconstruct out all the shared Orchestration props
    // and see if there's any props remaining
    if (transitionDefinition === undefined ||
        !isTransitionDefined(transitionDefinition)) {
        return __assign({ delay: delay }, getDefaultTransition(key, to));
    }
    var valueTransitionDefinition = transitionDefinition[key] ||
        transitionDefinition.default ||
        transitionDefinition;
    if (valueTransitionDefinition.type === false) {
        return {
            type: "just",
            delay: delay,
            to: isKeyframesTarget(to)
                ? to[to.length - 1]
                : to,
        };
    }
    else if (isKeyframesTarget(to)) {
        return __assign({ values: to, duration: 0.8, delay: delay, ease: "linear" }, valueTransitionDefinition, { 
            // This animation must be keyframes if we're animating through an array
            type: "keyframes" });
    }
    else {
        return __assign({ type: "tween", to: to,
            delay: delay }, valueTransitionDefinition);
    }
};
var preprocessOptions = function (type, opts) {
    return transitionOptionParser[type]
        ? transitionOptionParser[type](opts)
        : opts;
};
var getAnimation = function (key, value, target, transition) {
    var origin = value.get();
    var isOriginAnimatable = isAnimatable(key, origin);
    var isTargetAnimatable = isAnimatable(key, target);
    // TODO we could probably improve this check to ensure both values are of the same type -
    // for instance 100 to #fff. This might live better in Popmotion.
    heyListen.warning(isOriginAnimatable === isTargetAnimatable, "You are trying to animate " + key + " from \"" + origin + "\" to " + target + ". \"" + origin + "\" is not an animatable value - to enable this animation set " + origin + " to a value animatable to " + target + " via the `style` property.");
    // Parse the `transition` prop and return options for the Popmotion animation
    var _a = getTransitionDefinition(key, target, transition), _b = _a.type, type = _b === void 0 ? "tween" : _b, transitionDefinition = __rest(_a, ["type"]);
    // If this is an animatable pair of values, return an animation, otherwise use `just`
    var actionFactory = isOriginAnimatable && isTargetAnimatable ? transitions[type] : just;
    var opts = preprocessOptions(type, __assign({ from: origin, velocity: value.getVelocity() }, transitionDefinition));
    return [actionFactory, opts];
};
/**
 * Start animation on a value. This function completely encapsulates Popmotion-specific logic.
 *
 * @internal
 */
function startAnimation(key, value, target, _a) {
    var _b = _a.delay, delay = _b === void 0 ? 0 : _b, transition = __rest(_a, ["delay"]);
    var _c = getAnimation(key, value, target, transition), animationFactory = _c[0], opts = _c[1];
    // Convert durations from Framer Motion seconds into Popmotion milliseconds
    if (isDurationAnimation(opts) && opts.duration) {
        opts.duration *= 1000;
    }
    delay *= 1000;
    // Bind animation opts to animation
    var animation = animationFactory(opts);
    // Compose delay
    if (delay) {
        animation = popmotion.chain(popmotion.delay(delay), animation);
    }
    return value.start(function (complete) {
        var activeAnimation = animation.start({
            update: function (v) { return value.set(v); },
            complete: complete,
        });
        return function () { return activeAnimation.stop(); };
    });
}

/**
 * Get the current value of every `MotionValue`
 * @param values -
 */
var getCurrent = function (values) {
    var current = {};
    values.forEach(function (value, key) { return (current[key] = value.get()); });
    return current;
};
/**
 * Get the current velocity of every `MotionValue`
 * @param values -
 */
var getVelocity = function (values) {
    var velocity = {};
    values.forEach(function (value, key) { return (velocity[key] = value.getVelocity()); });
    return velocity;
};
/**
 * Check if value is a function that returns a `Target`. A generic typeof === 'function'
 * check, just helps with typing.
 * @param p -
 */
var isTargetResolver = function (p) {
    return typeof p === "function";
};
/**
 * Check if value is a list of variant labels
 * @param v -
 */
var isVariantLabels = function (v) { return Array.isArray(v); };
/**
 * Check if value is a numerical string, ie "100" or "100px"
 */
var isNumericalString = function (v) { return /^\d*\.?\d+$/.test(v); };
/**
 * Control animations for a single component
 * @internal
 */
var ValueAnimationControls = /** @class */ (function () {
    function ValueAnimationControls(_a) {
        var values = _a.values, readValueFromSource = _a.readValueFromSource, makeTargetAnimatable = _a.makeTargetAnimatable;
        var _this = this;
        /**
         * The component's variants, as provided by `variants`
         */
        this.variants = {};
        /**
         * A set of values that we animate back to when a value is cleared of all overrides.
         */
        this.baseTarget = {};
        /**
         * A series of target overrides that we can animate to/from when overrides are set/cleared.
         */
        this.overrides = [];
        /**
         * A series of target overrides as they were originally resolved.
         */
        this.resolvedOverrides = [];
        /**
         * A Set of currently active override indexes
         */
        this.activeOverrides = new Set();
        /**
         * A Set of value keys that are currently animating.
         */
        this.isAnimating = new Set();
        this.values = values;
        this.readValueFromSource = readValueFromSource;
        this.makeTargetAnimatable = makeTargetAnimatable;
        this.values.forEach(function (value, key) { return (_this.baseTarget[key] = value.get()); });
    }
    /**
     * Set the reference to the component's props.
     * @param props -
     */
    ValueAnimationControls.prototype.setProps = function (props) {
        this.props = props;
    };
    /**
     * Set the reference to the component's variants
     * @param variants -
     */
    ValueAnimationControls.prototype.setVariants = function (variants) {
        if (variants)
            this.variants = variants;
    };
    /**
     * Set the component's default transition
     * @param transition -
     */
    ValueAnimationControls.prototype.setDefaultTransition = function (transition) {
        if (transition)
            this.defaultTransition = transition;
    };
    /**
     * Set motion values without animation.
     *
     * @param target -
     * @param isActive -
     */
    ValueAnimationControls.prototype.setValues = function (target, isActive) {
        var _this = this;
        if (isActive === void 0) { isActive = new Set(); }
        target = this.transformValues(target);
        return Object.keys(target).forEach(function (key) {
            if (isActive.has(key))
                return;
            isActive.add(key);
            var targetValue = resolveFinalValueInKeyframes(target[key]);
            if (_this.values.has(key)) {
                var value = _this.values.get(key);
                value && value.set(targetValue);
            }
            else {
                _this.values.set(key, motionValue(targetValue));
            }
            _this.baseTarget[key] = targetValue;
        });
    };
    /**
     * Allows `transformValues` to be set by a component that allows us to
     * transform the values in a given `Target`. This allows Framer Library
     * to extend Framer Motion to animate `Color` variables etc. Currently we have
     * to manually support these extended types here in Framer Motion.
     *
     * @param values -
     */
    ValueAnimationControls.prototype.transformValues = function (values) {
        var transformValues = this.props.transformValues;
        return transformValues ? transformValues(values) : values;
    };
    /**
     * Check a `Target` for new values we haven't animated yet, and add them
     * to the `MotionValueMap`.
     *
     * Currently there's functionality here that is DOM-specific, we should allow
     * this functionality to be injected by the factory that creates DOM-specific
     * components.
     *
     * @param target -
     */
    ValueAnimationControls.prototype.checkForNewValues = function (target) {
        var _this = this;
        var newValueKeys = Object.keys(target).filter(function (key) { return !_this.values.has(key); });
        if (!newValueKeys.length)
            return;
        newValueKeys.forEach(function (key) {
            var value = _this.readValueFromSource(key);
            if (typeof value === "string" && isNumericalString(value)) {
                // If this is a number read as a string, ie "0" or "200", convert it to a number
                value = parseFloat(value);
            }
            else if (!getValueType(value) && styleValueTypes.complex.test(target[key])) {
                // If value is not recognised as animatable, ie "none", create an animatable version origin based on the target
                value = styleValueTypes.complex.getAnimatableNone(target[key]);
            }
            _this.values.set(key, motionValue(value));
            _this.baseTarget[key] = value;
        });
    };
    /**
     * Resolve a variant from its label or resolver into an actual `Target` we can animate to.
     * @param variant -
     */
    ValueAnimationControls.prototype.resolveVariant = function (variant) {
        if (!variant) {
            return {
                target: undefined,
                transition: undefined,
                transitionEnd: undefined,
            };
        }
        if (isTargetResolver(variant)) {
            // resolve current and velocity
            variant = variant(this.props.custom, getCurrent(this.values), getVelocity(this.values));
        }
        var _a = variant.transition, transition = _a === void 0 ? this.defaultTransition : _a, transitionEnd = variant.transitionEnd, target = __rest(variant, ["transition", "transitionEnd"]);
        return { transition: transition, transitionEnd: transitionEnd, target: target };
    };
    /**
     * Get the highest active override priority index
     */
    ValueAnimationControls.prototype.getHighestPriority = function () {
        if (!this.activeOverrides.size)
            return 0;
        return Math.max.apply(Math, Array.from(this.activeOverrides));
    };
    /**
     * Set an override. We add this layer of indirection so if, for instance, a tap gesture
     * starts and overrides a hover gesture, when we clear the tap gesture and fallback to the
     * hover gesture, if that hover gesture has changed in the meantime we can go to that rather
     * than the one that was resolved when the hover gesture animation started.
     *
     * @param definition -
     * @param overrideIndex -
     */
    ValueAnimationControls.prototype.setOverride = function (definition, overrideIndex) {
        this.overrides[overrideIndex] = definition;
        if (this.children) {
            this.children.forEach(function (child) {
                return child.setOverride(definition, overrideIndex);
            });
        }
    };
    /**
     * Start an override animation.
     * @param overrideIndex -
     */
    ValueAnimationControls.prototype.startOverride = function (overrideIndex) {
        var override = this.overrides[overrideIndex];
        if (override) {
            return this.start(override, { priority: overrideIndex });
        }
    };
    /**
     * Clear an override. We check every value we animated to in this override to see if
     * its present on any lower-priority overrides. If not, we animate it back to its base target.
     * @param overrideIndex -
     */
    ValueAnimationControls.prototype.clearOverride = function (overrideIndex) {
        if (this.children) {
            this.children.forEach(function (child) { return child.clearOverride(overrideIndex); });
        }
        var override = this.overrides[overrideIndex];
        if (!override)
            return;
        this.activeOverrides.delete(overrideIndex);
        var highest = this.getHighestPriority();
        this.resetIsAnimating();
        if (highest) {
            var highestOverride = this.overrides[highest];
            highestOverride && this.startOverride(highest);
        }
        // Figure out which remaining values were affected by the override and animate those
        var overrideTarget = this.resolvedOverrides[overrideIndex];
        if (!overrideTarget)
            return;
        var remainingValues = {};
        for (var key in this.baseTarget) {
            if (overrideTarget[key] !== undefined) {
                remainingValues[key] = this.baseTarget[key];
            }
        }
        this.animate(remainingValues);
    };
    /**
     * Apply a target/variant without any animation
     */
    ValueAnimationControls.prototype.apply = function (definition) {
        if (Array.isArray(definition)) {
            return this.applyVariantLabels(definition);
        }
        else if (typeof definition === "string") {
            return this.applyVariantLabels([definition]);
        }
        else {
            this.setValues(definition);
        }
    };
    /**
     * Apply variant labels without animation
     */
    ValueAnimationControls.prototype.applyVariantLabels = function (variantLabelList) {
        var _this = this;
        var isSetting = new Set();
        var reversedList = variantLabelList.slice().reverse();
        reversedList.forEach(function (key) {
            var _a = _this.resolveVariant(_this.variants[key]), target = _a.target, transitionEnd = _a.transitionEnd;
            if (transitionEnd) {
                _this.setValues(transitionEnd, isSetting);
            }
            if (target) {
                _this.setValues(target, isSetting);
            }
        });
    };
    ValueAnimationControls.prototype.start = function (definition, opts) {
        var _this = this;
        if (opts === void 0) { opts = {}; }
        if (opts.priority) {
            this.activeOverrides.add(opts.priority);
        }
        this.resetIsAnimating(opts.priority);
        var animation;
        if (isVariantLabels(definition)) {
            animation = this.animateVariantLabels(definition, opts);
        }
        else if (typeof definition === "string") {
            animation = this.animateVariant(definition, opts);
        }
        else {
            animation = this.animate(definition, opts);
        }
        return animation.then(function () {
            var onAnimationComplete = _this.props.onAnimationComplete;
            onAnimationComplete && onAnimationComplete();
        });
    };
    ValueAnimationControls.prototype.animate = function (animationDefinition, _a) {
        var _this = this;
        var _b = _a === void 0 ? {} : _a, _c = _b.delay, delay = _c === void 0 ? 0 : _c, _d = _b.priority, priority = _d === void 0 ? 0 : _d, transitionOverride = _b.transitionOverride;
        var _e = this.resolveVariant(animationDefinition), target = _e.target, transition = _e.transition, transitionEnd = _e.transitionEnd;
        if (transitionOverride) {
            transition = transitionOverride;
        }
        if (!target)
            return Promise.resolve();
        target = this.transformValues(target);
        if (transitionEnd) {
            transitionEnd = this.transformValues(transitionEnd);
        }
        if (this.makeTargetAnimatable) {
            var animatable = this.makeTargetAnimatable(target, transitionEnd);
            target = animatable.target;
            transitionEnd = animatable.transitionEnd;
        }
        if (priority) {
            this.resolvedOverrides[priority] = target;
        }
        this.checkForNewValues(target);
        var animations = Object.keys(target).reduce(function (acc, key) {
            var value = _this.values.get(key);
            if (!value || !target || target[key] === undefined)
                return acc;
            var valueTarget = target[key];
            if (!priority) {
                _this.baseTarget[key] = resolveFinalValueInKeyframes(valueTarget);
            }
            if (_this.isAnimating.has(key))
                return acc;
            acc.push(startAnimation(key, value, valueTarget, __assign({ delay: delay }, transition)));
            _this.isAnimating.add(key);
            return acc;
        }, []);
        return Promise.all(animations).then(function () {
            if (!transitionEnd)
                return;
            _this.setValues(transitionEnd);
        });
    };
    ValueAnimationControls.prototype.animateVariantLabels = function (variantLabels, opts) {
        var _this = this;
        var animations = variantLabels.slice().reverse()
            .map(function (label) { return _this.animateVariant(label, opts); });
        return Promise.all(animations);
    };
    ValueAnimationControls.prototype.animateVariant = function (variantLabel, opts) {
        var _this = this;
        var when = false;
        var delayChildren = 0;
        var staggerChildren = 0;
        var staggerDirection = 1;
        var priority = (opts && opts.priority) || 0;
        var variant = this.variants[variantLabel];
        var getAnimations = variant
            ? function () { return _this.animate(variant, opts); }
            : function () { return Promise.resolve(); };
        var getChildrenAnimations = this.children
            ? function () {
                return _this.animateChildren(variantLabel, delayChildren, staggerChildren, staggerDirection, priority);
            }
            : function () { return Promise.resolve(); };
        if (variant && this.children) {
            var transition = this.resolveVariant(variant).transition;
            if (transition) {
                when = transition.when || when;
                delayChildren = transition.delayChildren || delayChildren;
                staggerChildren = transition.staggerChildren || staggerChildren;
                staggerDirection =
                    transition.staggerDirection || staggerDirection;
            }
        }
        if (when) {
            var _a = when === "beforeChildren"
                ? [getAnimations, getChildrenAnimations]
                : [getChildrenAnimations, getAnimations], first = _a[0], last = _a[1];
            return first().then(last);
        }
        else {
            return Promise.all([getAnimations(), getChildrenAnimations()]);
        }
    };
    ValueAnimationControls.prototype.animateChildren = function (variantLabel, delayChildren, staggerChildren, staggerDirection, priority) {
        if (delayChildren === void 0) { delayChildren = 0; }
        if (staggerChildren === void 0) { staggerChildren = 0; }
        if (staggerDirection === void 0) { staggerDirection = 1; }
        if (priority === void 0) { priority = 0; }
        if (!this.children) {
            return Promise.resolve();
        }
        var animations = [];
        var maxStaggerDuration = (this.children.size - 1) * staggerChildren;
        var generateStaggerDuration = staggerDirection === 1
            ? function (i) { return i * staggerChildren; }
            : function (i) { return maxStaggerDuration - i * staggerChildren; };
        Array.from(this.children).forEach(function (childControls, i) {
            var animation = childControls.animateVariant(variantLabel, {
                priority: priority,
                delay: delayChildren + generateStaggerDuration(i),
            });
            animations.push(animation);
        });
        return Promise.all(animations);
    };
    ValueAnimationControls.prototype.checkOverrideIsAnimating = function (priority) {
        var numOverrides = this.overrides.length;
        for (var i = priority + 1; i < numOverrides; i++) {
            var resolvedOverride = this.resolvedOverrides[i];
            if (resolvedOverride) {
                for (var key in resolvedOverride) {
                    this.isAnimating.add(key);
                }
            }
        }
    };
    ValueAnimationControls.prototype.resetIsAnimating = function (priority) {
        if (priority === void 0) { priority = 0; }
        this.isAnimating.clear();
        // If this isn't the highest priority gesture, block the animation
        // of anything that's currently being animated
        if (priority < this.getHighestPriority()) {
            this.checkOverrideIsAnimating(priority);
        }
        if (this.children) {
            this.children.forEach(function (child) { return child.resetIsAnimating(priority); });
        }
    };
    ValueAnimationControls.prototype.stop = function () {
        this.values.forEach(function (value) { return value.stop(); });
    };
    /**
     * Add the controls of a child component.
     * @param controls -
     */
    ValueAnimationControls.prototype.addChild = function (controls) {
        if (!this.children) {
            this.children = new Set();
        }
        this.children.add(controls);
        // We set child overrides when `setOverride` is called, but also have to do it here
        // as the first time `setOverride` is called all the children might not have been added yet.
        this.overrides.forEach(function (override, i) {
            override && controls.setOverride(override, i);
        });
    };
    ValueAnimationControls.prototype.removeChild = function (controls) {
        if (!this.children) {
            return;
        }
        this.children.delete(controls);
    };
    ValueAnimationControls.prototype.resetChildren = function () {
        if (this.children)
            this.children.clear();
    };
    return ValueAnimationControls;
}());

/**
 * Use a callback a maximum number of times
 * @param callback - Callback to run
 * @param times - Maximum number of times to run the callback. Defaults to `1`
 *
 * @public
 */
function useMaxTimes(callback, times) {
    if (times === void 0) { times = 1; }
    var count = React.useRef(0);
    if (count.current++ < times)
        callback();
}

/**
 * @internal
 */
var MotionContext = React.createContext({
    static: false,
});
var isVariantLabel = function (v) {
    return typeof v === "string" || Array.isArray(v);
};
/**
 * Set up the context for children motion components.
 *
 * We also use this opportunity to apply `initial` values
 */
var useMotionContext = function (parentContext, controls, isStatic, _a) {
    if (isStatic === void 0) { isStatic = false; }
    var initial = _a.initial, animate = _a.animate, variants = _a.variants, whileTap = _a.whileTap, whileHover = _a.whileHover;
    // We pass on this component's ValueAnimationControls *if* we're being provided variants,
    // or if we're being used to control variants. Otherwise this component should be "invisible" to
    // variant propagation.
    var shouldPropagateControls = variants ||
        isVariantLabel(animate) ||
        isVariantLabel(whileTap) ||
        isVariantLabel(whileHover);
    // If this component's `initial` prop is a variant label, propagate it. Otherwise pass the parent's.
    var targetInitial = isVariantLabel(initial)
        ? initial
        : parentContext.initial;
    // Only allow `initial` to trigger context re-renders if this is a `static` component (ie we're on the Framer canvas)
    // or in another non-animation/interaction environment.
    var initialDependency = isStatic ? targetInitial : null;
    // Only allow `animate` to trigger context re-renders if it's a variant label. If this is an array of
    // variant labels there's probably an optimisation to deep-compare but it might be an over-optimisation.
    // We want to do this as we rely on React's component rendering order each render cycle to determine
    // the new order of any child components for the `staggerChildren` functionality.
    var animateDependency = isVariantLabel(animate) ? animate : null;
    // The context to provide to the child. We `useMemo` because although `controls` and `initial` are
    // unlikely to change, by making the context an object it'll be considered a new value every render.
    // So all child motion components will re-render as a result.
    var context = React.useMemo(function () { return ({
        controls: shouldPropagateControls
            ? controls
            : parentContext.controls,
        initial: targetInitial,
    }); }, [initialDependency, animateDependency]);
    // Update the `static` property every render. This is unlikely to change but also essentially free.
    context.static = isStatic;
    // Set initial state. If this is a static component (ie in Framer canvas), respond to updates
    // in `initial`.
    useMaxTimes(function () {
        var initialToApply = initial || parentContext.initial;
        initialToApply && controls.apply(initialToApply);
    }, isStatic ? Infinity : 1);
    return context;
};

/**
 * Creates an imperative set of controls to trigger animations.
 *
 * This allows a consolidated, uniform API for animations, to be triggered by other APIs like the `animate` prop, or the gesture handlers.
 *
 * @param values
 * @param props
 * @param ref
 * @param inheritVariantChanges
 *
 * @internal
 */
function useValueAnimationControls(config, props, inheritVariantChanges) {
    var variants = props.variants, transition = props.transition;
    var parentControls = React.useContext(MotionContext).controls;
    var controls = useConstant(function () { return new ValueAnimationControls(config); });
    // Reset and resubscribe children every render to ensure stagger order is correct
    controls.resetChildren();
    if (inheritVariantChanges && parentControls) {
        parentControls.addChild(controls);
    }
    React.useEffect(function () { return function () { return parentControls && parentControls.removeChild(controls); }; }, []);
    controls.setProps(props);
    controls.setVariants(variants);
    controls.setDefaultTransition(transition);
    return controls;
}

/**
 * Control animations on one or more components.
 *
 * @public
 */
var AnimationControls = /** @class */ (function () {
    function AnimationControls() {
        /**
         * Track whether the host component has mounted.
         *
         * @internal
         */
        this.hasMounted = false;
        /**
         * Pending animations that are started before a component is mounted.
         *
         * @internal
         */
        this.pendingAnimations = [];
        /**
         * A collection of linked component animation controls.
         *
         * @internal
         */
        this.componentControls = new Set();
    }
    /**
     * Set variants on this and all child components.
     *
     * @param variants - The variants to set
     *
     * @internal
     */
    AnimationControls.prototype.setVariants = function (variants) {
        this.variants = variants;
        this.componentControls.forEach(function (controls) {
            return controls.setVariants(variants);
        });
    };
    /**
     * Set a default transition on this and all child components
     *
     * @param transition - The default transition to set
     *
     * @internal
     */
    AnimationControls.prototype.setDefaultTransition = function (transition) {
        this.defaultTransition = transition;
        this.componentControls.forEach(function (controls) {
            return controls.setDefaultTransition(transition);
        });
    };
    /**
     * Subscribes a component's animation controls to this.
     *
     * @param controls - The controls to subscribe
     * @returns An unsubscribe function.
     *
     * @internal
     */
    AnimationControls.prototype.subscribe = function (controls) {
        var _this = this;
        this.componentControls.add(controls);
        if (this.variants)
            controls.setVariants(this.variants);
        if (this.defaultTransition)
            controls.setDefaultTransition(this.defaultTransition);
        return function () { return _this.componentControls.delete(controls); };
    };
    /**
     * Starts an animation on all linked components.
     *
     * @remarks
     *
     * ```jsx
     * animation.start("variantLabel")
     * animation.start({
     *   x: 0,
     *   transition: { duration: 1 }
     * })
     * ```
     *
     * @param definition - Properties or variant label to animate to
     * @param transition - Optional `transtion` to apply to a variant
     * @returns - A `Promise` that resolves when all animations have completed.
     *
     * @public
     */
    AnimationControls.prototype.start = function (definition, transitionOverride) {
        var _this = this;
        if (this.hasMounted) {
            var animations_1 = [];
            this.componentControls.forEach(function (controls) {
                var animation = controls.start(definition, {
                    transitionOverride: transitionOverride,
                });
                animations_1.push(animation);
            });
            return Promise.all(animations_1);
        }
        else {
            return new Promise(function (resolve) {
                _this.pendingAnimations.push({
                    animation: [definition, transitionOverride],
                    resolve: resolve,
                });
            });
        }
    };
    /**
     * Stops animations on all linked components.
     *
     * @public
     */
    AnimationControls.prototype.stop = function () {
        this.componentControls.forEach(function (controls) { return controls.stop(); });
    };
    /**
     * Initialises the animation controls.
     *
     * @internal
     */
    AnimationControls.prototype.mount = function () {
        var _this = this;
        this.hasMounted = true;
        this.pendingAnimations.forEach(function (_a) {
            var animation = _a.animation, resolve = _a.resolve;
            return _this.start.apply(_this, animation).then(resolve);
        });
    };
    /**
     * Stops all child animations when the host component unmounts.
     *
     * @internal
     */
    AnimationControls.prototype.unmount = function () {
        this.hasMounted = false;
        this.stop();
    };
    return AnimationControls;
}());
/**
 * @internal
 */
var animationControls = function () { return new AnimationControls(); };

var checkShouldInheritVariant = function (_a) {
    var animate = _a.animate, variants = _a.variants, _b = _a.inherit, inherit = _b === void 0 ? true : _b;
    return (inherit &&
        !!variants &&
        (!animate || animate instanceof AnimationControls));
};

var AnimatePropType;
(function (AnimatePropType) {
    AnimatePropType["Target"] = "Target";
    AnimatePropType["VariantLabel"] = "VariantLabel";
    AnimatePropType["AnimationSubscription"] = "AnimationSubscription";
})(AnimatePropType || (AnimatePropType = {}));

var makeHookComponent = function (hook) { return function (props) {
    hook(props);
    return null;
}; };

function shallowCompare(next, prev) {
    if (prev === null)
        return false;
    var prevLength = prev.length;
    if (prevLength !== next.length)
        return false;
    for (var i = 0; i < prevLength; i++) {
        if (prev[i] !== next[i])
            return false;
    }
    return true;
}

var hasUpdated = function (prev, next) {
    return (next !== undefined &&
        (Array.isArray(prev) && Array.isArray(next)
            ? !shallowCompare(next, prev)
            : prev !== next));
};
function justTarget(_a) {
    var transition = _a.transition, transitionEnd = _a.transitionEnd, target = __rest(_a, ["transition", "transitionEnd"]);
    return target;
}
/**
 * Handle the `animate` prop when its an object of values, ie:
 *
 * ```jsx
 * <motion.div animate={{ opacity: 1 }} />
 * ```
 *
 * @internalremarks
 * It might be worth consolidating this with `use-variants`
 *
 * ```jsx
 * <motion.div animate="visible" />
 * ```
 *
 * @param target
 * @param controls
 * @param values
 * @param transition
 *
 * @internal
 */
function useAnimateProp(targetAndTransition, controls, values, defaultTransition) {
    var isInitialRender = React.useRef(true);
    var prevValues = React.useRef(null);
    if (!prevValues.current) {
        prevValues.current = justTarget(targetAndTransition);
    }
    React.useEffect(function () {
        var targetToAnimate = {};
        var target = justTarget(targetAndTransition);
        // Detect which values have changed between renders
        for (var key in prevValues.current) {
            // This value should animate on mount if this value doesn't already exist (wasn't
            // defined in `style` or `initial`) or if it does exist and it's already changed.
            var shouldAnimateOnMount = isInitialRender.current &&
                (!values.has(key) || values.get(key) !== target[key]);
            // If this value has updated between renders or it's we're animating this value on mount,
            // add it to the animate target.
            if (hasUpdated(prevValues.current[key], target[key]) ||
                shouldAnimateOnMount) {
                targetToAnimate[key] = target[key];
            }
        }
        isInitialRender.current = false;
        prevValues.current = __assign({}, prevValues.current, target);
        if (Object.keys(targetToAnimate).length) {
            controls.start(__assign({}, targetToAnimate, { transition: targetAndTransition.transition || defaultTransition, transitionEnd: targetAndTransition.transitionEnd }));
        }
    }, [targetAndTransition]);
}

var labelsToArray = function (label) {
    if (!label) {
        return [];
    }
    if (Array.isArray(label)) {
        return label;
    }
    return [label];
};
var resolveVariantLabels = function (variant) {
    var unresolvedVariant = variant instanceof MotionValue ? variant.get() : variant;
    return Array.from(new Set(labelsToArray(unresolvedVariant)));
};
/**
 * Hooks in React sometimes accept a dependency array as their final argument. (ie useEffect/useMemo)
 * When values in this array change, React re-runs the dependency. However if the array
 * contains a variable number of items, React throws an error.
 */
var asDependencyList = function (list) { return [
    list.join(","),
]; };

var hasVariantChanged = function (oldVariant, newVariant) {
    return oldVariant.join(",") !== newVariant.join(",");
};
/**
 * Handle variants and the `animate` prop when its set as variant labels.
 *
 * @param targetVariant
 * @param inherit
 * @param controls
 * @param initialVariant
 *
 * @internal
 */
function useVariants(targetVariant, inherit, controls, initialVariant) {
    var variantList = resolveVariantLabels(targetVariant);
    var hasMounted = React.useRef(false);
    // Fire animations when poses change
    React.useEffect(function () {
        // TODO: This logic might mean we don't need to load this hook at all
        if (inherit)
            return;
        if (hasMounted.current ||
            hasVariantChanged(resolveVariantLabels(initialVariant), variantList)) {
            controls.start(variantList);
        }
        hasMounted.current = true;
    }, asDependencyList(variantList));
}

/**
 * `useAnimationGroupSubscription` allows a component to subscribe to an
 * externally-created `AnimationControls`, created by the `useAnimation` hook.
 *
 * @param animation
 * @param controls
 *
 * @internal
 */
function useAnimationGroupSubscription(animation, controls) {
    var unsubscribe = React.useMemo(function () { return animation.subscribe(controls); }, [
        animation,
    ]);
    React.useEffect(function () { return function () { return unsubscribe && unsubscribe(); }; }, [unsubscribe]);
}

var _a, _b;
var AnimatePropComponents = (_a = {},
    _a[AnimatePropType.Target] = makeHookComponent(function (_a) {
        var animate = _a.animate, controls = _a.controls, values = _a.values, transition = _a.transition;
        return useAnimateProp(animate, controls, values, transition);
    }),
    _a[AnimatePropType.VariantLabel] = makeHookComponent(function (_a) {
        var animate = _a.animate, _b = _a.inherit, inherit = _b === void 0 ? true : _b, controls = _a.controls, initial = _a.initial;
        return useVariants(animate, inherit, controls, initial);
    }),
    _a[AnimatePropType.AnimationSubscription] = makeHookComponent(function (_a) {
        var animate = _a.animate, controls = _a.controls;
        return useAnimationGroupSubscription(animate, controls);
    }),
    _a);
var isVariantLabel$1 = function (prop) {
    return Array.isArray(prop) || typeof prop === "string";
};
var isAnimationSubscription = function (_a) {
    var animate = _a.animate;
    return animate instanceof AnimationControls;
};
var animationProps = ["initial", "animate", "whileTap", "whileHover"];
var animatePropTypeTests = (_b = {},
    _b[AnimatePropType.Target] = function (props) {
        return (props.animate !== undefined &&
            !isVariantLabel$1(props.animate) &&
            !isAnimationSubscription(props));
    },
    _b[AnimatePropType.VariantLabel] = function (props) {
        return (props.variants !== undefined ||
            animationProps.some(function (key) { return typeof props[key] === "string"; }));
    },
    _b[AnimatePropType.AnimationSubscription] = isAnimationSubscription,
    _b);
var getAnimateComponent = function (props, isStatic) {
    if (isStatic === void 0) { isStatic = false; }
    var animatePropType = undefined;
    for (var key in AnimatePropType) {
        if (animatePropTypeTests[key](props)) {
            animatePropType = key;
        }
    }
    return !isStatic && animatePropType
        ? AnimatePropComponents[animatePropType]
        : undefined;
};

/**
 * @internal
 */
var createMotionComponent = function (_a) {
    var useFunctionalityComponents = _a.useFunctionalityComponents, getValueControlsConfig = _a.getValueControlsConfig;
    function MotionComponent(props, externalRef) {
        var ref = useExternalRef(externalRef);
        var parentContext = React.useContext(MotionContext);
        var isStatic = parentContext.static || props.static || false;
        var values = useMotionValues(props);
        var style = useMotionStyles(values, props.style, props.transformValues);
        var shouldInheritVariant = checkShouldInheritVariant(props);
        var controlsConfig = useConstant(function () {
            return getValueControlsConfig(ref, values);
        });
        var controls = useValueAnimationControls(controlsConfig, props, shouldInheritVariant);
        var context = useMotionContext(parentContext, controls, isStatic, props);
        // Add functionality
        var Animate = getAnimateComponent(props, context.static);
        var handleAnimate = Animate && (React.createElement(Animate, __assign({}, props, { inherit: shouldInheritVariant, innerRef: ref, values: values, controls: controls })));
        var handleActiveFunctionality = useFunctionalityComponents(props, values, controls, ref, style, context.static);
        return (React.createElement(MotionContext.Provider, { value: context },
            React.createElement(MountMotionValues, { ref: ref, values: values, isStatic: isStatic }),
            handleAnimate,
            handleActiveFunctionality));
    }
    return React.forwardRef(MotionComponent);
};

// These are identity functions that help TypeScript understand the type of the provided array
var htmlTuple = function (args) { return args; };
var svgTuple = function (args) { return args; };
/**
 * @internal
 */
var htmlElements = htmlTuple([
    "a",
    "abbr",
    "address",
    "area",
    "article",
    "aside",
    "audio",
    "b",
    "base",
    "bdi",
    "bdo",
    "big",
    "blockquote",
    "body",
    "br",
    "button",
    "canvas",
    "caption",
    "cite",
    "code",
    "col",
    "colgroup",
    "data",
    "datalist",
    "dd",
    "del",
    "details",
    "dfn",
    "dialog",
    "div",
    "dl",
    "dt",
    "em",
    "embed",
    "fieldset",
    "figcaption",
    "figure",
    "footer",
    "form",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
    "head",
    "header",
    "hgroup",
    "hr",
    "html",
    "i",
    "iframe",
    "img",
    "input",
    "ins",
    "kbd",
    "keygen",
    "label",
    "legend",
    "li",
    "link",
    "main",
    "map",
    "mark",
    "menu",
    "menuitem",
    "meta",
    "meter",
    "nav",
    "noscript",
    "object",
    "ol",
    "optgroup",
    "option",
    "output",
    "p",
    "param",
    "picture",
    "pre",
    "progress",
    "q",
    "rp",
    "rt",
    "ruby",
    "s",
    "samp",
    "script",
    "section",
    "select",
    "small",
    "source",
    "span",
    "strong",
    "style",
    "sub",
    "summary",
    "sup",
    "table",
    "tbody",
    "td",
    "textarea",
    "tfoot",
    "th",
    "thead",
    "time",
    "title",
    "tr",
    "track",
    "u",
    "ul",
    "var",
    "video",
    "wbr",
    "webview",
]);
/**
 * @internal
 */
var svgElements = svgTuple([
    "animate",
    "circle",
    "clipPath",
    "defs",
    "desc",
    "ellipse",
    "feBlend",
    "feColorMatrix",
    "feComponentTransfer",
    "feComposite",
    "feConvolveMatrix",
    "feDiffuseLighting",
    "feDisplacementMap",
    "feDistantLight",
    "feDropShadow",
    "feFlood",
    "feFuncA",
    "feFuncB",
    "feFuncG",
    "feFuncR",
    "feGaussianBlur",
    "feImage",
    "feMerge",
    "feMergeNode",
    "feMorphology",
    "feOffset",
    "fePointLight",
    "feSpecularLighting",
    "feSpotLight",
    "feTile",
    "feTurbulence",
    "filter",
    "foreignObject",
    "g",
    "image",
    "line",
    "linearGradient",
    "marker",
    "mask",
    "metadata",
    "path",
    "pattern",
    "polygon",
    "polyline",
    "radialGradient",
    "rect",
    "stop",
    "svg",
    "switch",
    "symbol",
    "text",
    "textPath",
    "tspan",
    "use",
    "view",
]);

function isEventTarget(target) {
    return (typeof target.addEventListener === "function" &&
        typeof target.removeEventListener === "function");
}
var eventListener = function (target, name, handler, options) {
    var startListening = function () {
        if (!target) {
            return;
        }
        // TODO: Pointer events are stacking after every pan start
        // https://github.com/framer/company/issues/12814
        target.addEventListener(name, handler, options);
    };
    var stopListening = function () {
        if (!target) {
            return;
        }
        target.removeEventListener(name, handler, options);
    };
    return [startListening, stopListening];
};
var useEvent = function (type, target, handler, options) {
    var result = undefined;
    if ((!target || isEventTarget(target)) && handler) {
        result = eventListener(target, type, handler, options);
    }
    React.useEffect(function () {
        var ref = target;
        if (!handler || !target || isEventTarget(target) || !ref.current) {
            return;
        }
        var _a = eventListener(ref.current, type, handler, options), start = _a[0], stop = _a[1];
        start();
        return stop;
    }, [type, target, handler, options]);
    return result;
};

var extractEventInfo = function (_a) {
    var _b = _a.pageX, pageX = _b === void 0 ? 0 : _b, _c = _a.pageY, pageY = _c === void 0 ? 0 : _c;
    return {
        point: { x: pageX, y: pageY },
    };
};
var wrapHandler = function (handler) {
    if (!handler) {
        return undefined;
    }
    var listener = function (event, info) {
        if (!info) {
            info = extractEventInfo(event);
        }
        handler(event, info);
    };
    return listener;
};

var mockWindow = {
    addEventListener: function () { },
    removeEventListener: function () { },
    dispatchEvent: function () { return false; },
    onpointerdown: false,
    onpointermove: false,
    onpointerup: false,
    ontouchstart: false,
    ontouchmove: false,
    ontouchend: false,
    onmousedown: false,
    onmousemove: false,
    onmouseup: false,
    devicePixelRatio: 1,
    scrollX: 0,
    scrollY: 0,
    location: {
        href: "",
    },
    setTimeout: function () { return 0; },
    clearTimeout: function () { },
    setInterval: function () { return 0; },
    clearInterval: function () { },
    webkitConvertPointFromPageToNode: function (_, pt) { return pt; },
};
/**
 * Creates a server-safe reference to `window`, returning a mock if none is available.
 *
 * @internal
 */
var safeWindow = typeof window === "undefined" ? mockWindow : window;

var noop = [function () { }, function () { }];
var mergeUseEventResults = function () {
    var values = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        values[_i] = arguments[_i];
    }
    if (values.every(function (v) { return v === undefined; })) {
        return noop;
    }
    var start = function () {
        for (var _i = 0, values_1 = values; _i < values_1.length; _i++) {
            var value = values_1[_i];
            if (value) {
                value[0]();
            }
        }
    };
    var stop = function () {
        for (var _i = 0, values_2 = values; _i < values_2.length; _i++) {
            var value = values_2[_i];
            if (value) {
                value[1]();
            }
        }
    };
    return [start, stop];
};
/**
 * Filters out events not attached to the primary pointer (currently left mouse button)
 * @param eventHandler
 */
var filterPrimaryPointer = function (eventHandler) {
    if (!eventHandler)
        return undefined;
    return function (event, info) {
        var isMouseEvent = event instanceof MouseEvent;
        var isPrimaryPointer = !isMouseEvent ||
            (isMouseEvent && event.button === 0);
        if (isPrimaryPointer) {
            eventHandler(event, info);
        }
    };
};
var useMouseEvents = function (_a, ref, options) {
    var onMouseDown = _a.onMouseDown, onMouseMove = _a.onMouseMove, onMouseUp = _a.onMouseUp, onMouseOver = _a.onMouseOver, onMouseOut = _a.onMouseOut, onMouseEnter = _a.onMouseEnter, onMouseLeave = _a.onMouseLeave;
    var down = useEvent("mousedown", ref, wrapHandler(filterPrimaryPointer(onMouseDown)), options);
    var move = useEvent("mousemove", ref, wrapHandler(onMouseMove), options);
    var up = useEvent("mouseup", ref, wrapHandler(onMouseUp), options);
    var over = useEvent("mouseover", ref, wrapHandler(onMouseOver), options);
    var out = useEvent("mouseout", ref, wrapHandler(onMouseOut), options);
    var enter = useEvent("mouseenter", ref, wrapHandler(onMouseEnter), options);
    var leave = useEvent("mouseleave", ref, wrapHandler(onMouseLeave), options);
    return mergeUseEventResults(down, move, up, over, out, enter, leave);
};
var useTouchEvents = function (_a, ref, options) {
    var onTouchStart = _a.onTouchStart, onTouchMove = _a.onTouchMove, onTouchEnd = _a.onTouchEnd, onTouchCancel = _a.onTouchCancel;
    var down = useEvent("touchstart", ref, wrapHandler(onTouchStart), options);
    var move = useEvent("touchmove", ref, wrapHandler(onTouchMove), options);
    var up = useEvent("touchend", ref, wrapHandler(onTouchEnd), options);
    var cancel = useEvent("touchcancel", ref, wrapHandler(onTouchCancel), options);
    return mergeUseEventResults(down, move, up, cancel);
};
var useNativePointerEvents = function (_a, ref, options) {
    var onPointerDown = _a.onPointerDown, onPointerMove = _a.onPointerMove, onPointerUp = _a.onPointerUp, onPointerCancel = _a.onPointerCancel, onPointerOver = _a.onPointerOver, onPointerOut = _a.onPointerOut, onPointerEnter = _a.onPointerEnter, onPointerLeave = _a.onPointerLeave;
    var down = useEvent("pointerdown", ref, wrapHandler(filterPrimaryPointer(onPointerDown)), options);
    var move = useEvent("pointermove", ref, wrapHandler(onPointerMove), options);
    var up = useEvent("pointerup", ref, wrapHandler(onPointerUp), options);
    var cancel = useEvent("pointercancel", ref, wrapHandler(onPointerCancel), options);
    var over = useEvent("pointerover", ref, wrapHandler(onPointerOver), options);
    var out = useEvent("pointerout", ref, wrapHandler(onPointerOut), options);
    var enter = useEvent("pointerenter", ref, wrapHandler(onPointerEnter), options);
    var leave = useEvent("pointerleave", ref, wrapHandler(onPointerLeave), options);
    return mergeUseEventResults(down, move, up, cancel, over, out, enter, leave);
};
var supportsPointerEvents = function () {
    return safeWindow.onpointerdown === null &&
        safeWindow.onpointermove === null &&
        safeWindow.onpointerup === null;
};
var supportsTouchEvents = function () {
    return safeWindow.ontouchstart === null &&
        safeWindow.ontouchmove === null &&
        safeWindow.ontouchend === null;
};
var supportsMouseEvents = function () {
    return safeWindow.onmousedown === null &&
        safeWindow.onmousemove === null &&
        safeWindow.onmouseup === null;
};
var usePointerEvents = function (eventHandlers, ref, options) {
    var onPointerDown = eventHandlers.onPointerDown, onPointerMove = eventHandlers.onPointerMove, onPointerUp = eventHandlers.onPointerUp, onPointerCancel = eventHandlers.onPointerCancel, onPointerOver = eventHandlers.onPointerOver, onPointerOut = eventHandlers.onPointerOut, onPointerEnter = eventHandlers.onPointerEnter, onPointerLeave = eventHandlers.onPointerLeave;
    var mouseEvents = {}, touchEvents = {}, pointerEvents = {};
    if (supportsPointerEvents()) {
        pointerEvents = eventHandlers;
    }
    else if (supportsTouchEvents()) {
        touchEvents = {
            onTouchStart: onPointerDown,
            onTouchMove: onPointerMove,
            onTouchEnd: onPointerUp,
            onTouchCancel: onPointerCancel,
        };
    }
    else if (supportsMouseEvents()) {
        mouseEvents = {
            onMouseDown: onPointerDown,
            onMouseMove: onPointerMove,
            onMouseUp: onPointerUp,
            onMouseOver: onPointerOver,
            onMouseOut: onPointerOut,
            onMouseEnter: onPointerEnter,
            onMouseLeave: onPointerLeave,
        };
    }
    var pointer = useNativePointerEvents(pointerEvents, ref, options);
    var touch = useTouchEvents(touchEvents, ref, options);
    var mouse = useMouseEvents(mouseEvents, ref, options);
    return mergeUseEventResults(pointer, touch, mouse);
};
/**
 * A Conditional version of usePointerEvents
 * As opposed to usePointerEvents, this function does not require to pass a target or ref.
 * When calling this without a target, the handlers passed to the function are returned
 * @param eventHandlers
 * @param ref
 * @param options
 */
var useConditionalPointerEvents = function (eventHandlers, ref, options) {
    var pointerEventsRef;
    var pointerEventsHandlers;
    var emptyRef = React.useRef(null);
    // Because hooks can't be conditional, we need to this dance, instead of just an early return
    if (ref) {
        pointerEventsRef = ref;
        pointerEventsHandlers = eventHandlers;
    }
    else {
        pointerEventsRef = emptyRef;
        pointerEventsHandlers = {};
    }
    usePointerEvents(pointerEventsHandlers, pointerEventsRef, options);
    if (ref) {
        return undefined;
    }
    else {
        return eventHandlers;
    }
};

/** @public */
(function (Point) {
    /** @beta */
    Point.subtract = function (a, b) {
        return { x: a.x - b.x, y: a.y - b.y };
    };
    /** @beta */
    Point.relativeTo = function (idOrElem) {
        var elem;
        var getElem = function () {
            // Caching element here could be leaky because of React lifecycle
            if (elem !== undefined)
                return elem;
            if (typeof idOrElem === "string") {
                elem = document.getElementById(idOrElem);
            }
            else {
                elem = idOrElem;
            }
            return elem;
        };
        return function (_a) {
            var x = _a.x, y = _a.y;
            var localElem = getElem();
            if (!localElem)
                return undefined;
            var rect = localElem.getBoundingClientRect();
            return {
                x: x - rect.left - window.scrollX,
                y: y - rect.top - window.scrollY,
            };
        };
    };
})(exports.Point || (exports.Point = {}));

/**
 * @internal
 */
var MotionPluginContext = React.createContext({});
/**
 * @internal
 * @internalremarks For now I think this should remain a private API for our own use
 * until we can figure out a nicer way of allowing people to add these
 */
function MotionPlugins(_a) {
    var children = _a.children, props = __rest(_a, ["children"]);
    var pluginContext = React.useContext(MotionPluginContext);
    var value = React.useRef(__assign({}, pluginContext)).current;
    // Mutative to prevent triggering rerenders in all listening
    // components every time this component renders
    for (var key in props) {
        value[key] = props[key];
    }
    return (React.createElement(MotionPluginContext.Provider, { value: value }, children));
}

var isViewportScrollBlocked = false;
safeWindow.addEventListener("touchmove", function (event) {
    if (isViewportScrollBlocked) {
        event.preventDefault();
    }
}, { passive: false });
var blockViewportScroll = function () { return (isViewportScrollBlocked = true); };
var unblockViewportScroll = function () { return (isViewportScrollBlocked = false); };

function startDevicePoint(session) {
    return session.pointHistory[0];
}
function lastDevicePoint(session) {
    return session.pointHistory[session.pointHistory.length - 1];
}
function getVelocity$1(session, timeDelta) {
    var pointHistory = session.pointHistory;
    if (pointHistory.length < 2) {
        return { x: 0, y: 0 };
    }
    var i = pointHistory.length - 1;
    var timestampedPoint = null;
    var lastPoint = lastDevicePoint(session);
    while (i >= 0) {
        timestampedPoint = pointHistory[i];
        if (lastPoint.timestamp - timestampedPoint.timestamp >
            timeDelta * 1000) {
            break;
        }
        i--;
    }
    if (!timestampedPoint) {
        return { x: 0, y: 0 };
    }
    var time = (lastPoint.timestamp - timestampedPoint.timestamp) / 1000;
    if (time === 0) {
        return { x: 0, y: 0 };
    }
    var currentVelocity = {
        x: (lastPoint.x - timestampedPoint.x) / time,
        y: (lastPoint.y - timestampedPoint.y) / time,
    };
    if (currentVelocity.x === Infinity) {
        currentVelocity.x = 0;
    }
    if (currentVelocity.y === Infinity) {
        currentVelocity.y = 0;
    }
    return currentVelocity;
}
function usePanGesture(_a, ref) {
    var onPan = _a.onPan, onPanStart = _a.onPanStart, onPanEnd = _a.onPanEnd, onPanSessionStart = _a.onPanSessionStart;
    var session = React.useRef(null);
    var pointer = React.useRef(null);
    var lastMoveEvent = React.useRef(null);
    var lastMoveEventInfo = React.useRef(null);
    var transformPagePoint = React.useContext(MotionPluginContext).transformPagePoint;
    var getPanInfo = function (_a) {
        var point = _a.point;
        var currentPoint = session.current;
        return {
            point: point,
            delta: exports.Point.subtract(point, lastDevicePoint(currentPoint)),
            offset: exports.Point.subtract(point, startDevicePoint(currentPoint)),
            velocity: getVelocity$1(currentPoint, 0.1),
        };
    };
    var transformPoint = function (info) {
        return transformPagePoint
            ? { point: transformPagePoint(info.point) }
            : info;
    };
    var updatePoint = React.useCallback(function () {
        if (!session.current ||
            pointer.current === null ||
            lastMoveEventInfo.current === null ||
            lastMoveEvent.current === null) {
            heyListen.warning(false, "Pointer move without started session");
            stopPointerMove();
            stopPointerUp();
            unblockViewportScroll();
            return;
        }
        var info = getPanInfo(lastMoveEventInfo.current);
        var point = info.point;
        var timestamp = sync.getFrameData().timestamp;
        session.current.pointHistory.push(__assign({}, point, { timestamp: timestamp }));
        pointer.current.x.set(point.x);
        pointer.current.y.set(point.y);
        if (!session.current.startEvent) {
            if (onPanStart) {
                onPanStart(lastMoveEvent.current, info);
            }
            session.current.startEvent = lastMoveEvent.current;
        }
        if (onPan) {
            onPan(lastMoveEvent.current, info);
        }
    }, [onPan, onPanStart]);
    var onPointerMove = React.useCallback(function (event, info) {
        lastMoveEvent.current = event;
        lastMoveEventInfo.current = transformPoint(info);
        // because Safari doesn't trigger mouseup event when it's happening above <select> tag
        if (event instanceof MouseEvent && event.buttons === 0) {
            onPointerUp(event, info);
            return;
        }
        // Throttle mouse move event to once per frame
        sync__default.update(updatePoint, true);
    }, [onPan, onPanStart]);
    var onPointerUp = React.useCallback(function (event, info) {
        sync.cancelSync.update(updatePoint);
        stopPointerMove();
        stopPointerUp();
        if (!session.current || pointer.current === null) {
            heyListen.warning(false, "Pointer end without started session");
            unblockViewportScroll();
            return;
        }
        if (onPanEnd) {
            onPanEnd(event, getPanInfo(transformPoint(info)));
        }
        session.current = null;
    }, [onPanEnd, onPointerMove]);
    var _b = usePointerEvents({ onPointerUp: onPointerUp }, safeWindow), startPointerUp = _b[0], stopPointerUp = _b[1];
    var _c = usePointerEvents({ onPointerMove: onPointerMove }, safeWindow, { capture: true }), startPointerMove = _c[0], stopPointerMove = _c[1];
    var onPointerDown = React.useCallback(function (event, info) {
        var initialInfo = transformPoint(info);
        var point = initialInfo.point;
        pointer.current = {
            x: motionValue(point.x),
            y: motionValue(point.y),
        };
        var timestamp = sync.getFrameData().timestamp;
        session.current = {
            target: event.target,
            pointHistory: [__assign({}, point, { timestamp: timestamp })],
        };
        if (onPanSessionStart) {
            onPanSessionStart(event, getPanInfo(initialInfo));
        }
        startPointerMove();
        startPointerUp();
    }, [onPointerUp, onPointerMove]);
    React.useEffect(function () {
        return function () {
            stopPointerMove();
            stopPointerUp();
        };
    }, []);
    var handlers = { onPointerDown: onPointerDown };
    if (!(onPan || onPanStart || onPanEnd || onPanSessionStart)) {
        handlers = {};
    }
    return useConditionalPointerEvents(handlers, ref);
}

/**
 * Recursively traverse up the tree to check whether the provided child node
 * is the parent or a descendant of it.
 *
 * @param parent - Element to find
 * @param child - Element to test against parent
 */
var isNodeOrChild = function (parent, child) {
    if (!child) {
        return false;
    }
    else if (parent === child) {
        return true;
    }
    else {
        return isNodeOrChild(parent, child.parentElement);
    }
};

var order = ["whileHover", "whileTap", "whileDrag"];
var getGesturePriority = function (gesture) {
    return order.indexOf(gesture) + 1;
};

function createLock(name) {
    var lock = null;
    return function () {
        var openLock = function () {
            lock = null;
        };
        if (lock === null) {
            lock = name;
            return openLock;
        }
        return false;
    };
}
var globalHorizontalLock = createLock("dragHorizontal");
var globalVerticalLock = createLock("dragVertical");
function getGlobalLock(drag) {
    var lock = false;
    if (drag === "y") {
        lock = globalVerticalLock();
    }
    else if (drag === "x") {
        lock = globalHorizontalLock();
    }
    else {
        var openHorizontal_1 = globalHorizontalLock();
        var openVertical_1 = globalVerticalLock();
        if (openHorizontal_1 && openVertical_1) {
            lock = function () {
                openHorizontal_1();
                openVertical_1();
            };
        }
        else {
            // Release the locks because we don't use them
            if (openHorizontal_1)
                openHorizontal_1();
            if (openVertical_1)
                openVertical_1();
        }
    }
    return lock;
}

var tapGesturePriority = getGesturePriority("whileTap");
/**
 * Explanation:
 *
 * Currently there's a problem where props can change while a gesture is active. In
 * this hook, we have event handlers that are created in `useMemo`, which is sync.
 * The event handlers themselves are activated in `useEvent` with `useEffect`, which
 * is async.
 *
 * Most event handlers are being set like this: onTap={() => {}}
 * This creates a new function every render, thus breaking `useMemo`. When any props
 * are set during a gesture, this creates a new event. We then have a `useEffect` in this
 * hook that, when props change, unsets the previous event handlers. I suspect that at
 * some point, the pointer up event is fired in between the previous one
 * being deactivate and the new one being activated.
 *
 * I attempted to rewrite the hook without `useMemo` at all but encountered the same issue.
 * At some point the `pointerUp` event is being unset and not reapplied until after it fires.
 *
 * By setting props to a ref we maintain longer gesture sessions and don't encounter this funny
 * business, but I'd rather a solution that required less code.
 *
 * @param props
 */
var usePropsRef = function (props) {
    var propsRef = React.useRef(props);
    for (var key in props) {
        propsRef.current[key] = props[key];
    }
    return propsRef.current;
};
function useTapGesture(props, ref) {
    var isTapping = React.useRef(false);
    var onTap = props.onTap, onTapStart = props.onTapStart, onTapCancel = props.onTapCancel, whileTap = props.whileTap, controls = props.controls;
    var propsRef = usePropsRef(props);
    if (whileTap && controls) {
        controls.setOverride(whileTap, tapGesturePriority);
    }
    var handlers = React.useMemo(function () {
        if (!onTap && !onTapStart && !onTapCancel && !whileTap) {
            return {
                onPointerUp: function () { },
                onPointerDown: function () { },
            };
        }
        var onPointerUp = function (event, _a) {
            var point = _a.point;
            if (!isTapping.current) {
                return;
            }
            isTapping.current = false;
            if (controls && propsRef.whileTap) {
                controls.clearOverride(tapGesturePriority);
            }
            // Check the gesture lock - if we get it, it means no drag gesture is active
            // and we can safely fire the tap gesture.
            var openGestureLock = getGlobalLock(true);
            if (!openGestureLock) {
                return;
            }
            openGestureLock();
            if (!ref ||
                !ref.current ||
                !isNodeOrChild(ref.current, event.target)) {
                if (propsRef.onTapCancel) {
                    propsRef.onTapCancel(event, { point: point });
                }
                return;
            }
            if (propsRef.onTap) {
                propsRef.onTap(event, { point: point });
            }
        };
        var onPointerDown = function (event, _a) {
            var point = _a.point;
            startPointerUp();
            if (!ref || !ref.current)
                return;
            isTapping.current = true;
            if (propsRef.onTapStart) {
                propsRef.onTapStart(event, { point: point });
            }
            if (controls && propsRef.whileTap) {
                controls.startOverride(tapGesturePriority);
            }
        };
        return { onPointerUp: onPointerUp, onPointerDown: onPointerDown };
    }, [
        onTap === undefined,
        onTapStart === undefined,
        onTapCancel === undefined,
    ]);
    var _a = usePointerEvents({ onPointerUp: handlers.onPointerUp }, safeWindow), startPointerUp = _a[0], stopPointerUp = _a[1];
    React.useEffect(function () { return function () {
        stopPointerUp();
    }; }, [handlers.onPointerUp]);
    return useConditionalPointerEvents({ onPointerDown: handlers.onPointerDown }, ref);
}

var hoverPriority = getGesturePriority("whileHover");
// TODO: Optimisation here is find a way to conditionally add these listeners based on
// whether we're receiving hover or event listeners
/**
 *
 * @param props
 * @param ref
 * @internal
 */
function useHoverGesture(_a, ref) {
    var whileHover = _a.whileHover, onHoverStart = _a.onHoverStart, onHoverEnd = _a.onHoverEnd, controls = _a.controls;
    if (whileHover && controls) {
        controls.setOverride(whileHover, hoverPriority);
    }
    var handlers = React.useMemo(function () {
        var onPointerEnter = function (event) {
            if (onHoverStart)
                onHoverStart(event);
            if (whileHover && controls) {
                controls.startOverride(hoverPriority);
            }
        };
        var onPointerLeave = function (event) {
            if (onHoverEnd)
                onHoverEnd(event);
            if (whileHover && controls) {
                controls.clearOverride(hoverPriority);
            }
        };
        return { onPointerEnter: onPointerEnter, onPointerLeave: onPointerLeave };
    }, [whileHover, onHoverStart, onHoverEnd, controls]);
    return useConditionalPointerEvents(handlers, ref);
}

/**
 * Add pan and tap gesture recognition to an element.
 *
 * @param props - Gesture event handlers
 * @param ref - React `ref` containing a DOM `Element`
 * @public
 */
function useGestures(props, ref) {
    usePanGesture(props, ref);
    useTapGesture(props, ref);
    useHoverGesture(props, ref);
}

var gestureProps = [
    "drag",
    "onPan",
    "onPanStart",
    "onPanEnd",
    "onPanSessionStart",
    "onTap",
    "onTapStart",
    "onTapCancel",
    "whileTap",
    "whileHover",
    "onHoverStart",
    "onHoverEnd",
];
var Gestures = {
    shouldRender: function (props) {
        return gestureProps.some(function (key) { return props.hasOwnProperty(key); });
    },
    component: makeHookComponent(function (_a) {
        var innerRef = _a.innerRef, props = __rest(_a, ["innerRef"]);
        useGestures(props, innerRef);
    }),
};

var isRefObject = function (ref) {
    return typeof ref === "object" && ref.hasOwnProperty("current");
};

/**
 * A hook to use the window resize listener. In future it might be cool to use `ResizeObserver`
 * but it currently needs to be polyfilled and the size trade-off isn't there for this use-case.
 *
 * TODO: Debounce this bad boy
 *
 * @param onResize
 */
function useResize(element, onResize) {
    React.useEffect(function () {
        if (!element || !isRefObject(element))
            return;
        safeWindow.addEventListener("resize", onResize);
        return function () { return safeWindow.removeEventListener("resize", onResize); };
    }, [element, onResize]);
}

var getBoundingBox = function (ref) {
    return ref.current.getBoundingClientRect();
};
var getCurrentOffset = function (point) {
    return point ? point.get() : 0;
};
/**
 * Takes a parent Element and a draggable Element and returns pixel-based drag constraints.
 *
 * @param constraintsRef
 * @param draggableRef
 */
var calculateConstraintsFromDom = function (constraintsRef, draggableRef, point) {
    heyListen.invariant(constraintsRef.current !== null && draggableRef.current !== null, "If `dragConstraints` is set as a React ref, that ref must be passed to another component's `ref` prop.");
    var parentBoundingBox = getBoundingBox(constraintsRef);
    var draggableBoundingBox = getBoundingBox(draggableRef);
    var top = parentBoundingBox.top -
        draggableBoundingBox.top +
        getCurrentOffset(point.y);
    var left = parentBoundingBox.left -
        draggableBoundingBox.left +
        getCurrentOffset(point.x);
    var constraints = {
        top: top,
        left: left,
        right: parentBoundingBox.width - draggableBoundingBox.width + left,
        bottom: parentBoundingBox.height - draggableBoundingBox.height + top,
    };
    return constraints;
};
var flattenConstraints = function (constraints) {
    if (!constraints) {
        return [0, 0, 0, 0];
    }
    else if (isRefObject(constraints)) {
        return [constraints.current];
    }
    else {
        var top_1 = constraints.top, left = constraints.left, bottom = constraints.bottom, right = constraints.right;
        return [top_1, left, bottom, right];
    }
};
function shouldDrag(direction, drag, currentDirection) {
    return ((drag === true || drag === direction) &&
        (currentDirection === null || currentDirection === direction));
}
var getConstraints = function (axis, _a) {
    var top = _a.top, right = _a.right, bottom = _a.bottom, left = _a.left;
    if (axis === "x") {
        return { min: left, max: right };
    }
    else {
        return { min: top, max: bottom };
    }
};
function applyConstraints(axis, value, constraints, dragElastic) {
    var constrainedValue = value instanceof MotionValue ? value.get() : value;
    if (!constraints) {
        return constrainedValue;
    }
    var _a = getConstraints(axis, constraints), min = _a.min, max = _a.max;
    if (min !== undefined && constrainedValue < min) {
        constrainedValue = dragElastic
            ? applyOverdrag(min, constrainedValue, dragElastic)
            : Math.max(min, constrainedValue);
    }
    else if (max !== undefined && constrainedValue > max) {
        constrainedValue = dragElastic
            ? applyOverdrag(max, constrainedValue, dragElastic)
            : Math.min(max, constrainedValue);
    }
    if (value instanceof MotionValue) {
        value.set(constrainedValue);
    }
    return constrainedValue;
}
var applyOverdrag = function (origin, current, dragElastic) {
    var dragFactor = typeof dragElastic === "number" ? dragElastic : 0.35;
    return popcorn.mix(origin, current, dragFactor);
};
var bothAxis = function (handler) { return [
    handler("x"),
    handler("y"),
]; };
/**
 * A hook that allows an element to be dragged.
 *
 * @internalremarks
 *
 * TODO:
 *  - When drag momentum animations are running and a `ref` constraints is resized,
 *    everything breaks.
 *
 * @param param
 * @param ref
 * @param values
 * @param controls
 *
 * @internal
 */
function useDraggable(_a, ref, values, controls) {
    var _b = _a.drag, drag = _b === void 0 ? false : _b, _c = _a.dragDirectionLock, dragDirectionLock = _c === void 0 ? false : _c, _d = _a.dragPropagation, dragPropagation = _d === void 0 ? false : _d, _e = _a.dragConstraints, dragConstraints = _e === void 0 ? false : _e, _f = _a.dragElastic, dragElastic = _f === void 0 ? true : _f, _g = _a.dragMomentum, dragMomentum = _g === void 0 ? true : _g, dragTransition = _a.dragTransition, handlers = __rest(_a, ["drag", "dragDirectionLock", "dragPropagation", "dragConstraints", "dragElastic", "dragMomentum", "dragTransition"]);
    var point = React.useRef({}).current;
    var origin = React.useRef({ x: 0, y: 0 }).current;
    // By keeping a reference to the user-defined drag handlers and referring
    // to that from within our `useMemo`-generated pan handlers, we can ensure
    // we're always referring to the latest ones *without* having to reinitialise
    // the handlers. These are almost guaranteed to be different every render
    // but don't necessitate creating new handlers.
    var dragHandlers = React.useRef(handlers);
    dragHandlers.current = handlers;
    // If `dragConstraints` is a React `ref`, we should resolve the constraints once the
    // component has rendered.
    var constraintsNeedResolution = isRefObject(dragConstraints);
    // If `dragConstraints` is a React `ref`, we need to track changes in its
    // size and update the current draggable position relative to that.
    var prevConstraintsBox = React.useRef({
        width: 0,
        height: 0,
        x: 0,
        y: 0,
    }).current;
    var scalePoint = function () {
        if (!isRefObject(dragConstraints))
            return;
        var constraintsBox = getBoundingBox(dragConstraints);
        var draggableBox = getBoundingBox(ref);
        // Scale a point relative to the transformation of a constraints-providing element.
        var scaleAxisPoint = function (axis, dimension) {
            var pointToScale = point[axis];
            if (!pointToScale)
                return;
            // Stop any current animations as they bug out if you resize during one
            if (pointToScale.isAnimating()) {
                pointToScale.stop();
                recordBoxInfo();
                return;
            }
            // If the previous dimension was `0` (default), set `scale` to `1` to prevent
            // divide by zero errors.
            var scale = prevConstraintsBox[dimension]
                ? (constraintsBox[dimension] - draggableBox[dimension]) /
                    prevConstraintsBox[dimension]
                : 1;
            pointToScale.set(prevConstraintsBox[axis] * scale);
        };
        scaleAxisPoint("x", "width");
        scaleAxisPoint("y", "height");
    };
    useResize(dragConstraints, scalePoint);
    // If our drag constraints are a potentially live bounding box, record its previously-calculated
    // dimensions and the current x/y
    var recordBoxInfo = function (constraints) {
        if (constraints) {
            var right = constraints.right, left = constraints.left, bottom = constraints.bottom, top_2 = constraints.top;
            prevConstraintsBox.width = (right || 0) - (left || 0);
            prevConstraintsBox.height = (bottom || 0) - (top_2 || 0);
        }
        if (point.x)
            prevConstraintsBox.x = point.x.get();
        if (point.y)
            prevConstraintsBox.y = point.y.get();
    };
    var applyConstraintsToPoint = function (constraints) {
        return bothAxis(function (axis) {
            var axisPoint = point[axis];
            axisPoint && applyConstraints(axis, axisPoint, constraints, 0);
        });
    };
    // On mount, if our bounding box is a ref, we need to resolve the constraints
    // and immediately apply them to our point.
    React.useEffect(function () {
        if (!constraintsNeedResolution)
            return;
        var constraints = calculateConstraintsFromDom(dragConstraints, ref, point);
        applyConstraintsToPoint(constraints);
        recordBoxInfo(constraints);
    });
    // Create our handlers for the `pan` gesture.
    var panHandlers = React.useMemo(function () {
        if (!drag)
            return {};
        // We'll use this to determine whether to fire the `onDragEnd` callback.
        var hasDragged = false;
        // Don't start dragging until we've detected a direction.
        var currentDirection = null;
        // This is a reference to the global drag gesture lock, ensuring only one component
        // can "capture" the drag of one or both axes. In some odd circumstances, a re-render
        // has caused reference to this lock to be lost, if we see this appear again it might
        // be safer to move this to a hook-root `ref`.
        var openGlobalLock = null;
        // If `dragConstraints` is set to `false` or `Constraints`, set constraints immediately.
        // Otherwise we'll resolve on mount.
        var resolvedDragConstraints = constraintsNeedResolution
            ? false
            : dragConstraints;
        // Get the `MotionValue` for both draggable axes, or create them if they don't already
        // exist on this component.
        bothAxis(function (axis) {
            if (!shouldDrag(axis, drag, currentDirection))
                return;
            var axisValue = values.get(axis, 0);
            point[axis] = axisValue;
        });
        // Apply constraints immediately, even before render, if our constraints are a plain object.
        if (resolvedDragConstraints && !constraintsNeedResolution) {
            applyConstraintsToPoint(resolvedDragConstraints);
        }
        // Add additional information to the `PanInfo` object before passing it to drag listeners.
        var convertPanToDrag = function (info) { return (__assign({}, info, { point: {
                x: point.x ? point.x.get() : 0,
                y: point.y ? point.y.get() : 0,
            } })); };
        // This function will be used to update each axis point every frame.
        var updatePoint = function (axis, offset) {
            var axisPoint = point[axis];
            // If we're not dragging this axis, do an early return.
            if (!shouldDrag(axis, drag, currentDirection) || !axisPoint) {
                return;
            }
            hasDragged = true;
            var current = origin[axis] + offset[axis];
            current = applyConstraints(axis, current, resolvedDragConstraints, dragElastic);
            axisPoint.set(current);
        };
        var onPointerDown = function () {
            // Initiate viewport scroll blocking on touch start. This is a very aggressive approach
            // which has come out of the difficulty in us being able to do this once a scroll gesture
            // has initiated in mobile browsers. This means if there's a horizontally-scrolling carousel
            // on a page we can let a user scroll the page itself from it. Ideally what we'd do is
            // trigger this once we've got a scroll direction determined. This approach sort-of worked
            // but if the component was dragged quite far in a single frame page scrolling would initiate.
            // Maybe if we turn the direction lock threshold down.
            blockViewportScroll();
            // Stop any animations on both axis values immediately. This allows the user to throw and catch
            // the component.
            bothAxis(function (axis) {
                var axisPoint = point[axis];
                axisPoint && axisPoint.stop();
            });
        };
        var onPanStart = function (event, info) {
            hasDragged = false;
            // Resolve the constraints again in case anything has changed in the meantime.
            if (constraintsNeedResolution) {
                resolvedDragConstraints = calculateConstraintsFromDom(dragConstraints, ref, point);
                applyConstraintsToPoint(resolvedDragConstraints);
            }
            // Set point origin and stop any existing animations.
            bothAxis(function (axis) {
                var axisPoint = point[axis];
                if (!axisPoint)
                    return;
                origin[axis] = axisPoint.get();
                axisPoint.stop();
            });
            // Attempt to grab the global drag gesture lock.
            if (!dragPropagation) {
                openGlobalLock = getGlobalLock(drag);
                if (!openGlobalLock) {
                    return;
                }
            }
            currentDirection = null;
            // Alert listeners that dragging has started.
            var onDragStart = dragHandlers.current.onDragStart;
            onDragStart && onDragStart(event, convertPanToDrag(info));
        };
        var onPan = function (event, info) {
            // If we didn't successfully receive the gesture lock, early return.
            if (!dragPropagation && !openGlobalLock) {
                return;
            }
            var offset = info.offset;
            // Attempt to detect drag direction if directionLock is true
            if (dragDirectionLock && currentDirection === null) {
                currentDirection = getCurrentDirection(offset);
                // If we've successfully set a direction, notify listener
                if (currentDirection !== null) {
                    var onDirectionLock = dragHandlers.current.onDirectionLock;
                    onDirectionLock && onDirectionLock(currentDirection);
                }
                return;
            }
            updatePoint("x", offset);
            updatePoint("y", offset);
            var onDrag = dragHandlers.current.onDrag;
            onDrag && onDrag(event, convertPanToDrag(info));
        };
        var onPanEnd = function (event, info) {
            unblockViewportScroll();
            var velocity = info.velocity;
            if (!dragPropagation && openGlobalLock) {
                openGlobalLock();
            }
            else if (!openGlobalLock) {
                return;
            }
            if (!hasDragged)
                return;
            // If we have `dragMomentum` defined, initiate momentum animations for both axis.
            if (dragMomentum) {
                var momentumAnimations = bothAxis(function (axis) {
                    var _a;
                    if (!shouldDrag(axis, drag, currentDirection)) {
                        return;
                    }
                    var transition = resolvedDragConstraints
                        ? getConstraints(axis, resolvedDragConstraints)
                        : {};
                    return controls.start((_a = {},
                        _a[axis] = 0,
                        // TODO: It might be possible to allow `type` animations to be set as
                        // Popmotion animations as well as strings. Then people could define their own
                        // and it'd open another route for us to code-split.
                        _a.transition = __assign({ type: "inertia", velocity: velocity[axis], bounceStiffness: 200, bounceDamping: 40, timeConstant: 750, restDelta: 1 }, dragTransition, transition),
                        _a));
                });
                // Run all animations and then resolve the new drag constraints.
                Promise.all(momentumAnimations).then(function () {
                    recordBoxInfo(resolvedDragConstraints);
                    scalePoint();
                    var onDragTransitionEnd = dragHandlers.current.onDragTransitionEnd;
                    onDragTransitionEnd && onDragTransitionEnd();
                });
            }
            else {
                recordBoxInfo(resolvedDragConstraints);
            }
            var onDragEnd = dragHandlers.current.onDragEnd;
            onDragEnd && onDragEnd(event, convertPanToDrag(info));
        };
        return {
            onPanStart: onPanStart,
            onPan: onPan,
            onPanEnd: onPanEnd,
            onPointerDown: onPointerDown,
        };
    }, [
        drag,
        dragDirectionLock,
        dragPropagation,
        dragElastic,
        dragMomentum
    ].concat(flattenConstraints(dragConstraints), [
        dragTransition,
    ]));
    usePanGesture(panHandlers, ref);
    usePointerEvents({ onPointerDown: panHandlers.onPointerDown }, ref);
}
/**
 * Based on an x/y offset determine the current drag direction. If both axis' offsets are lower
 * than the provided threshold, return `null`.
 *
 * @param offset - The x/y offset from origin.
 * @param lockThreshold - (Optional) - the minimum absolute offset before we can determine a drag direction.
 */
function getCurrentDirection(offset, lockThreshold) {
    if (lockThreshold === void 0) { lockThreshold = 10; }
    var direction = null;
    if (Math.abs(offset.y) > lockThreshold) {
        direction = "y";
    }
    else if (Math.abs(offset.x) > lockThreshold) {
        direction = "x";
    }
    return direction;
}

var Drag = {
    shouldRender: function (props) { return !!props.drag; },
    component: makeHookComponent(function (_a) {
        var innerRef = _a.innerRef, values = _a.values, controls = _a.controls, props = __rest(_a, ["innerRef", "values", "controls"]);
        return useDraggable(props, innerRef, values, controls);
    }),
};

function isCSSVariable(value) {
    return typeof value === "string" && value.startsWith("var(--");
}
/**
 * Parse Framer's special CSS variable format into a CSS token and a fallback.
 *
 * ```
 * `var(--foo, #fff)` => [`--foo`, '#fff']
 * ```
 *
 * @param current
 */
var cssVariableRegex = /var\((--[a-zA-Z0-9-_]+),? ?([a-zA-Z0-9 ()%#.,-]+)?\)/;
function parseCSSVariable(current) {
    var match = cssVariableRegex.exec(current);
    if (!match)
        return [,];
    var token = match[1], fallback = match[2];
    return [token, fallback];
}
var maxDepth = 4;
function getVariableValue(current, element, depth) {
    if (depth === void 0) { depth = 1; }
    heyListen.invariant(depth <= maxDepth, "Max CSS variable fallback depth detected in property \"" + current + "\". This may indicate a circular fallback dependency.");
    var _a = parseCSSVariable(current), token = _a[0], fallback = _a[1];
    // No CSS variable detected
    if (!token)
        return;
    // Attempt to read this CSS variable off the element
    var resolved = window.getComputedStyle(element).getPropertyValue(token);
    if (resolved) {
        return resolved;
    }
    else if (isCSSVariable(fallback)) {
        // The fallback might itself be a CSS variable, in which case we attempt to resolve it too.
        return getVariableValue(fallback, element, depth + 1);
    }
    else {
        return fallback;
    }
}
/**
 * Resolve CSS variables from
 *
 * @internal
 */
function resolveCSSVariables(values, ref, _a, transitionEnd) {
    var target = __rest(_a, []);
    var element = ref.current;
    if (!(element instanceof HTMLElement))
        return { target: target, transitionEnd: transitionEnd };
    // If `transitionEnd` isn't `undefined`, clone it. We could clone `target` and `transitionEnd`
    // only if they change but I think this reads clearer and this isn't a performance-critical path.
    if (transitionEnd) {
        transitionEnd = __assign({}, transitionEnd);
    }
    // Go through existing `MotionValue`s and ensure any existing CSS variables are resolved
    values.forEach(function (value) {
        var current = value.get();
        if (!isCSSVariable(current))
            return;
        var resolved = getVariableValue(current, element);
        if (resolved)
            value.set(resolved);
    });
    // Cycle through every target property and resolve CSS variables. Currently
    // we only read single-var properties like `var(--foo)`, not `calc(var(--foo) + 20px)`
    for (var key in target) {
        var current = target[key];
        if (!isCSSVariable(current))
            continue;
        var resolved = getVariableValue(current, element);
        if (!resolved)
            continue;
        // Clone target if it hasn't already been
        target[key] = resolved;
        // If the user hasn't already set this key on `transitionEnd`, set it to the unresolved
        // CSS variable. This will ensure that after the animation the component will reflect
        // changes in the value of the CSS variable.
        if (transitionEnd && transitionEnd[key] === undefined) {
            transitionEnd[key] = current;
        }
    }
    return { target: target, transitionEnd: transitionEnd };
}

var positionalKeys = new Set([
    "width",
    "height",
    "top",
    "left",
    "right",
    "bottom",
    "x",
    "y",
]);
var isPositionalKey = function (key) { return positionalKeys.has(key); };
var hasPositionalKey = function (target) {
    return Object.keys(target).some(isPositionalKey);
};
var setAndResetVelocity = function (value, to) {
    // Looks odd but setting it twice doesn't render, it'll just
    // set both prev and current to the latest value
    value.set(to, false);
    value.set(to);
};
var isNumOrPxType = function (v) {
    return v === styleValueTypes.number || v === styleValueTypes.px;
};
var BoundingBoxDimension;
(function (BoundingBoxDimension) {
    BoundingBoxDimension["width"] = "width";
    BoundingBoxDimension["height"] = "height";
    BoundingBoxDimension["left"] = "left";
    BoundingBoxDimension["right"] = "right";
    BoundingBoxDimension["top"] = "top";
    BoundingBoxDimension["bottom"] = "bottom";
})(BoundingBoxDimension || (BoundingBoxDimension = {}));
var getPosFromMatrix = function (matrix, pos) {
    return parseFloat(matrix.split(", ")[pos]);
};
var getTranslateFromMatrix = function (pos2, pos3) { return function (_bbox, _a) {
    var transform = _a.transform;
    if (transform === "none" || !transform)
        return 0;
    var matrix3d = transform.match(/^matrix3d\((.+)\)$/);
    if (matrix3d) {
        return getPosFromMatrix(matrix3d[1], pos3);
    }
    else {
        var matrix = transform.match(/^matrix\((.+)\)$/);
        return getPosFromMatrix(matrix[1], pos2);
    }
}; };
var positionalValues = {
    // Dimensions
    width: function (_a) {
        var width = _a.width;
        return width;
    },
    height: function (_a) {
        var height = _a.height;
        return height;
    },
    top: function (_bbox, _a) {
        var top = _a.top;
        return parseFloat(top);
    },
    left: function (_bbox, _a) {
        var left = _a.left;
        return parseFloat(left);
    },
    bottom: function (_a, _b) {
        var height = _a.height;
        var top = _b.top;
        return parseFloat(top) + height;
    },
    right: function (_a, _b) {
        var width = _a.width;
        var left = _b.left;
        return parseFloat(left) + width;
    },
    // Transform
    x: getTranslateFromMatrix(4, 13),
    y: getTranslateFromMatrix(5, 14),
};
var convertChangedValueTypes = function (target, values, ref, changedKeys) {
    var element = ref.current;
    var elementStyler = styler__default(element);
    var originBbox = element.getBoundingClientRect();
    var elementComputedStyle = getComputedStyle(element);
    var top = elementComputedStyle.top, left = elementComputedStyle.left, bottom = elementComputedStyle.bottom, right = elementComputedStyle.right, transform = elementComputedStyle.transform;
    var originComputedStyle = { top: top, left: left, bottom: bottom, right: right, transform: transform };
    // Apply the latest values (as set in checkAndConvertChangedValueTypes)
    elementStyler.render();
    var targetBbox = element.getBoundingClientRect();
    changedKeys.forEach(function (key) {
        // Restore styles to their **calculated computed style**, not their actual
        // originally set style. This allows us to animate between equivalent pixel units.
        var value = values.get(key);
        setAndResetVelocity(value, positionalValues[key](originBbox, originComputedStyle));
        target[key] = positionalValues[key](targetBbox, elementComputedStyle);
    });
    // Reapply original values
    elementStyler.render();
    return target;
};
var checkAndConvertChangedValueTypes = function (values, ref, target, transitionEnd) {
    if (transitionEnd === void 0) { transitionEnd = {}; }
    target = __assign({}, target);
    transitionEnd = __assign({}, transitionEnd);
    var targetPositionalKeys = Object.keys(target).filter(isPositionalKey);
    var changedValueTypeKeys = targetPositionalKeys.reduce(function (acc, key) {
        var value = values.get(key);
        if (!value)
            return acc;
        var from = value.get();
        var to = target[key];
        var fromType = getDimensionValueType(from);
        var toType;
        // TODO: The current implementation of this basically throws an error
        // if you try and do value conversion via keyframes. There's probably
        // a way of doing this but the performance implications would need greater scrutiny,
        // as it'd be doing multiple resize-remeasure operations.
        if (isKeyframesTarget(to)) {
            var numKeyframes = to.length;
            for (var i = to[0] === null ? 1 : 0; i < numKeyframes; i++) {
                if (!toType) {
                    toType = getDimensionValueType(to[i]);
                    heyListen.invariant(toType === fromType ||
                        (isNumOrPxType(fromType) &&
                            isNumOrPxType(toType)), "Keyframes must be of the same dimension as the current value");
                }
                else {
                    heyListen.invariant(getDimensionValueType(to[i]) === toType, "All keyframes must be of the same type");
                }
            }
        }
        else {
            toType = getDimensionValueType(to);
        }
        if (fromType !== toType) {
            // If they're both just number or px, convert them both to numbers rather than
            // relying on resize/remeasure to convert (which is wasteful in this situation)
            if (isNumOrPxType(fromType) && isNumOrPxType(toType)) {
                var current = value.get();
                if (typeof current === "string") {
                    value.set(parseFloat(current));
                }
                if (typeof to === "string") {
                    target[key] = parseFloat(to);
                }
                else if (Array.isArray(to) && toType === styleValueTypes.px) {
                    target[key] = to.map(parseFloat);
                }
            }
            else {
                acc.push(key);
                transitionEnd[key] =
                    transitionEnd[key] !== undefined
                        ? transitionEnd[key]
                        : target[key];
                setAndResetVelocity(value, to);
            }
        }
        return acc;
    }, []);
    return changedValueTypeKeys.length
        ? {
            target: convertChangedValueTypes(target, values, ref, changedValueTypeKeys),
            transitionEnd: transitionEnd,
        }
        : { target: target, transitionEnd: transitionEnd };
};
/**
 * Convert value types for x/y/width/height/top/left/bottom/right
 *
 * Allows animation between `'auto'` -> `'100%'` or `0` -> `'calc(50% - 10vw)'`
 *
 * @param values
 * @param ref
 * @param target
 * @param transitionEnd
 * @internal
 */
function unitConversion(values, ref, target, transitionEnd) {
    return hasPositionalKey(target)
        ? checkAndConvertChangedValueTypes(values, ref, target, transitionEnd)
        : { target: target, transitionEnd: transitionEnd };
}

var parseDomVariant = function (values, ref) {
    return function (target, transitionEnd) {
        var resolved = resolveCSSVariables(values, ref, target, transitionEnd);
        target = resolved.target;
        transitionEnd = resolved.transitionEnd;
        return unitConversion(values, ref, target, transitionEnd);
    };
};

/**
 * Maintain a list of event handlers. Emotion's `is-prop-valid` doesn't whitelist every event
 * handler, it just pattern-matches `onX`. So we can manually strip out ours.
 */
var eventHandlers = new Set([
    "onAnimationComplete",
    "onUpdate",
    "onDragStart",
    "onDrag",
    "onDragEnd",
    "onDirectionLock",
    "onDragTransitionEnd"
].concat(gestureProps));
/**
 * Returns an object of only valid props. We remove invalid DOM props according to
 * Emotion's `isPropValid`, and any of our own event handlers. This was necessary when
 * we passed all a component's props through to dynamic variant resolvers, but since
 * moving that logic to custom it's possible for us to ditch this and save some filesize/performance
 * by simply destructuring out the props that we use.
 */
var validProps = function (props) {
    var valid = {};
    for (var key in props) {
        if (isPropValid(key) && !eventHandlers.has(key)) {
            valid[key] = props[key];
        }
    }
    return valid;
};
/**
 * Create a configuration for `motion` components that provides DOM-specific functionality.
 *
 * @internal
 */
function createDomMotionConfig(Component) {
    var isDOM = typeof Component === "string";
    var isSVG = isDOM && svgElements.indexOf(Component) !== -1;
    /**
     * Create a component that renders the DOM element. This step of indirection
     * could probably be removed at this point, and `createElement` could be moved
     * to the `activeComponents.push`.
     */
    var RenderComponent = function (_a) {
        var innerRef = _a.innerRef, style = _a.style, values = _a.values, isStatic = _a.isStatic, componentProps = _a.componentProps;
        var forwardProps = isDOM ? validProps(componentProps) : componentProps;
        return React.createElement(Component, __assign({}, forwardProps, { ref: innerRef, style: isSVG ? style : buildStyleAttr(values, style, isStatic) }));
    };
    return {
        /**
         * This hook gets used by the `motion` component
         *
         * Each functionality component gets provided the `ref`, animation controls and the `MotionValuesMap`
         * generated for that component, as well as all the `props` passed to it by the user.
         *
         * The pattern used to determine whether to load and use each piece of functionality is
         * consistent (should render? Then push component) and could be used to extend functionality.
         *
         * By exposing a mutable piece of memory via an API like `extendMotionComponent` we could
         * allow users to add `FunctionalComponentDefinition`s. This would allow us to offer file size
         * reductions by shipping an entry point that doesn't load gesture and drag functionality, and
         * also offer a way for users to develop plugins/other functionality.
         *
         * For user-defined functionality we'd need to allow
         *  1) User-defined prop typing (extending `P`)
         *  2) User-defined "clean props" that removes their plugin's props before being passed to the DOM.
         */
        useFunctionalityComponents: function (props, values, controls, ref, style, isStatic) {
            var activeComponents = [];
            if (!isStatic && Drag.shouldRender(props)) {
                activeComponents.push(React.createElement(Drag.component, __assign({}, props, { values: values, controls: controls, innerRef: ref, key: "drag" })));
            }
            if (!isStatic && Gestures.shouldRender(props)) {
                activeComponents.push(React.createElement(Gestures.component, __assign({}, props, { values: values, controls: controls, innerRef: ref, key: "gestures" })));
            }
            activeComponents.push(React.createElement(RenderComponent, { componentProps: props, values: values, controls: controls, innerRef: ref, style: style, isStatic: isStatic, key: "renderComponent" }));
            return activeComponents;
        },
        getValueControlsConfig: function (ref, values) {
            return {
                values: values,
                readValueFromSource: function (key) {
                    return styler__default(ref.current).get(key);
                },
                // TODO: This is a good second source of plugins. This function contains the CSS variable
                // and unit conversion support. These functions share a common signature. We could make another
                // API for adding these and extend to support stuff like FLIP
                makeTargetAnimatable: parseDomVariant(values, ref),
            };
        },
    };
}

var htmlMotionComponents = htmlElements.reduce(function (acc, Component) {
    acc[Component] = createMotionComponent(createDomMotionConfig(Component));
    return acc;
}, {});
var svgMotionComponents = svgElements.reduce(function (acc, Component) {
    acc[Component] = createMotionComponent(createDomMotionConfig(Component));
    return acc;
}, {});
/**
 * HTML & SVG components, optimised for use with gestures and animation. These can be used as
 * drop-in replacements for any HTML & SVG component, all CSS & SVG properties are supported.
 *
 * @internalremarks
 *
 * I'd like to make it possible for these to be loaded "on demand" - to reduce bundle size by only
 * including HTML/SVG stylers, animation and/or gesture support when necessary.
 *
 * ```jsx
 * <motion.div animate={{ x: 100 }} />
 *
 * <motion.p animate={{ height: 200 }} />
 *
 * <svg><motion.circle r={10} animate={{ r: 20 }} /></svg>
 * ```
 *
 * @public
 */
var motion = __assign({ custom: function (Component) {
        return createMotionComponent(createDomMotionConfig(Component));
    } }, htmlMotionComponents, svgMotionComponents);

/**
 * Creates a `MotionValue` to track the state and velocity of a value.
 *
 * Usually, these are created automatically. For advanced use-cases, like use with `useTransform`, you can create `MotionValue`s externally and pass them into the animated component via the `style` prop.
 *
 * ```jsx
 * export function MyComponent() {
 *   const scale = useMotionValue(1)
 *
 *   return <Frame style={{ scale: scale }} />
 * }
 * ```
 *
 * @param initial - The initial state.
 *
 * @public
 */
function useMotionValue(initial) {
    return useConstant(function () { return motionValue(initial); });
}

/**
 * If the provided value is a MotionValue, this returns the actual value, otherwise just the value itself
 *
 * TODO: Remove and move to library
 *
 * @internal
 */
function unwrapMotionValue(value) {
    var unwrappedValue = value instanceof MotionValue ? value.get() : value;
    return isCustomValue(unwrappedValue)
        ? unwrappedValue.toValue()
        : unwrappedValue;
}

function useCustomValue(parent, getTransformer, comparitor) {
    var value = React.useRef(null);
    return React.useMemo(function () {
        if (value.current)
            value.current.destroy();
        value.current = parent.addChild({ transformer: getTransformer() });
        return value.current;
    }, comparitor);
}

var isCustomValueType = function (v) {
    return typeof v === "object" && v.mix;
};
var getMixer = function (v) { return (isCustomValueType(v) ? v.mix : undefined); };
function transform() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var useImmediate = !Array.isArray(args[0]);
    var argOffset = useImmediate ? 0 : -1;
    var inputValue = args[0 + argOffset];
    var inputRange = args[1 + argOffset];
    var outputRange = args[2 + argOffset];
    var options = args[3 + argOffset];
    var interpolator = popcorn.interpolate(inputRange, outputRange, __assign({ mixer: getMixer(outputRange[0]) }, options));
    return useImmediate ? interpolator(inputValue) : interpolator;
}

var isTransformer = function (v) {
    return typeof v === "function";
};
var noop$1 = function () { return function (v) { return v; }; };
function useTransform(value, customTransform, to, options) {
    var comparitor = [value];
    var transformer = noop$1;
    if (isTransformer(customTransform)) {
        transformer = function () { return customTransform; };
    }
    else if (Array.isArray(to)) {
        var from_1 = customTransform;
        transformer = function () { return transform(from_1, to, options); };
        comparitor = [value, from_1.join(","), to.join(",")];
    }
    return useCustomValue(value, transformer, comparitor);
}

var scrollX = motionValue(0);
var scrollY = motionValue(0);
var scrollXProgress = motionValue(0);
var scrollYProgress = motionValue(0);
var setProgress = function (offset, maxOffset, value) {
    value.set(!maxOffset || !offset ? 0 : offset / maxOffset);
};
var hasEventListener = false;
var addScrollListener = function () {
    hasEventListener = true;
    if (typeof window === "undefined")
        return;
    var updateScrollValues = function () {
        var xOffset = window.pageXOffset;
        var yOffset = window.pageYOffset;
        // Set absolute positions
        scrollX.set(xOffset);
        scrollY.set(yOffset);
        // Set 0-1 progress
        setProgress(xOffset, document.body.clientWidth - window.innerWidth, scrollXProgress);
        setProgress(yOffset, document.body.clientHeight - window.innerHeight, scrollYProgress);
    };
    updateScrollValues();
    window.addEventListener("resize", updateScrollValues);
    window.addEventListener("scroll", updateScrollValues, { passive: true });
};
var viewportMotionValues = {
    scrollX: scrollX,
    scrollY: scrollY,
    scrollXProgress: scrollXProgress,
    scrollYProgress: scrollYProgress,
};
/**
 * Provides a `MotionValue` that updates when the viewport scrolls.
 * Returns the following four values.
 *
 * - `scrollX` — Horizontal scroll distance in pixels.
 * - `scrollY` — Vertical scroll distance in pixels.
 * - `scrollXProgress` — Horizontal scroll distance between `0` and `1`.
 * - `scrollYProgress` — Vertical scroll distance between `0` and `1`.
 *
 *
 * ```jsx
 * import * as React from "react"
 * import {
 *   Frame,
 *   useViewportScroll,
 *   useTransform
 * } from "framer"
 *
 * export function MyComponent() {
 *   const { scrollYProgress } = useViewportScroll()
 *   return <Frame style={{ scaleX: scrollYProgress }} />
 * }
 * ```
 *
 * @public
 */
function useViewportScroll() {
    if (!hasEventListener) {
        addScrollListener();
    }
    return viewportMotionValues;
}

/**
 * The `useAnimation` hook returns `AnimationControls`, which can be used to manually start, stop and sequence animations on one or more Frames.
 *
 * The returned `AnimationControls` should be passed as the `animate` property of the `Frame`. It can be passed to any number of Frames. These Frames can be animated with the `start` method. It accepts the same properties as the `animate` property.
 *
 * ```jsx
 *
 * import * as React from "react"
 * import { Frame, useAnimation } from "framer"
 *
 * export function MyComponent(props) {
 *    const controls = useAnimation()
 *
 *    controls.start({
 *        x: 100,
 *        transition: { duration: 0.5 },
 *    })
 *
 *    return <Frame animate={controls} />
 * }
 *
 * ```
 *
 * @returns Animation controller with `start` and `stop` methods
 *
 * @public
 */
function useAnimation() {
    var animationControls$$1 = useConstant(function () { return new AnimationControls(); });
    React.useEffect(function () {
        animationControls$$1.mount();
        return function () { return animationControls$$1.unmount(); };
    }, []);
    return animationControls$$1;
}

/**
 * Makes an animated version of `useState`.
 *
 * @remarks
 *
 * When the returned state setter is called, values will be animated to their new target.
 *
 * This allows the animation of arbitrary React components.
 *
 * **Note:** When animating DOM components, it's always preferable to use the `animate` prop, as Framer
 * will bypass React's rendering cycle with one optimised for 60fps motion. This Hook is specifically
 * for animating props on arbitrary React components, or for animating text content.
 *
 * ```jsx
 * const [state, setState] = useAnimatedState({ percentage: 0 })
 *
 * return (
 *   <Graph
 *     percentage={state.percentage}
 *     onTap={() => setState({ percentage: 50 })}
 *   />
 * )
 * ```
 *
 * @internalremarks
 *
 * TODO:
 * - Make hook accept a typed version of Target that accepts any value (not just DOM values)
 * - Allow hook to accept single values. ie useAnimatedState(0)
 * - Allow providing MotionValues via initialState.
 *
 * @beta
 */
function useAnimatedState(initialState) {
    var _a = React.useState(initialState), animationState = _a[0], onUpdate = _a[1];
    var config = useConstant(function () { return ({ onUpdate: onUpdate }); });
    var values = useMotionValues(config);
    var controls = useValueAnimationControls({
        values: values,
        readValueFromSource: function (key) { return animationState[key]; },
    }, {}, false);
    var startAnimation = useConstant(function () { return function (animationDefinition) {
        return controls.start(animationDefinition);
    }; });
    React.useEffect(function () {
        values.mount();
        return function () { return values.unmount(); };
    });
    return [animationState, startAnimation];
}

/**
 * Cycles through a series of visual properties. Can be used to toggle between or cycle through animations. It works similar to `useState` in React. It is provided an initial array of possible states, and returns an array of two arguments.
 *
 *
 * ```jsx
 * import * as React from "react"
 * import { Frame, useCycle } from "framer"
 *
 * export function MyComponent() {
 *   const [x, cycleX] = useCycle(0, 50, 100)
 *
 *   return (
 *     <Frame
 *       animate={{ x: x }}
 *       onTap={() => cycleX()}
 *      />
 *    )
 * }
 * ```
 *
 * @param items - items to cycle through
 * @returns [currentState, cycleState]
 *
 * @public
 */
function useCycle() {
    var items = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        items[_i] = arguments[_i];
    }
    // TODO: After Framer X beta, remove this warning
    heyListen.warning(items.length > 1, "useCycle syntax has changed. `useCycle([0, 1, 2])` becomes `useCycle(0, 1, 2)`");
    var index = React.useRef(0);
    var _a = React.useState(items[index.current]), item = _a[0], setItem = _a[1];
    return [
        item,
        function (next) {
            index.current =
                typeof next !== "number"
                    ? popcorn.wrap(0, items.length, index.current + 1)
                    : next;
            setItem(items[index.current]);
        },
    ];
}

exports.motion = motion;
exports.useExternalRef = useExternalRef;
exports.createMotionComponent = createMotionComponent;
exports.useMotionValue = useMotionValue;
exports.MotionValue = MotionValue;
exports.motionValue = motionValue;
exports.unwrapMotionValue = unwrapMotionValue;
exports.useTransform = useTransform;
exports.useViewportScroll = useViewportScroll;
exports.AnimationControls = AnimationControls;
exports.animationControls = animationControls;
exports.useAnimation = useAnimation;
exports.useAnimatedState = useAnimatedState;
exports.useTapGesture = useTapGesture;
exports.usePanGesture = usePanGesture;
exports.useGestures = useGestures;
exports.useCycle = useCycle;
exports.transform = transform;
exports.MotionPlugins = MotionPlugins;
exports.MotionPluginContext = MotionPluginContext;
exports.MotionContext = MotionContext;
exports.safeWindow = safeWindow;
